import org.gradle.kotlin.dsl.support.uppercaseFirstChar
import java.io.File

private enum class ChatEngineDependency {
    MOCK,
    TOX,
    MATRIX,
}

fun File.writeKoinModule(
    packageName: String,
    isMatrixEnabled: Boolean,
    isMockEngineEnabled: Boolean,
) {
    parentFile.mkdirs()
    val sourceCode = buildString {
        line("package $packageName\n")

        // Imports
        val engines = buildSet {
            add(ChatEngineDependency.TOX)
            if (isMatrixEnabled) add(ChatEngineDependency.MATRIX)
            if (isMockEngineEnabled) add(ChatEngineDependency.MOCK)
        }
        engines.forEach { engine ->
            val prefix = engine.name.lowercase()
            line("import nand.apps.chat.engine.${prefix}.${engine.function}")
            if (engine == ChatEngineDependency.MATRIX) {
                line("import nand.apps.chat.engine.matrix.image.MatrixImageFetcher")
            }
        }
        line("import org.koin.core.module.Module")
        line("import org.koin.core.Koin")
        line("import coil3.ComponentRegistry")
        line()

        // Koin modules
        line("fun getChatEngineKoinModules(): List<Module> = listOf(")
        engines.forEach { engine ->
            line("${engine.function}(),", 1)
        }
        line(")")

        // Image loaders
        line("fun ComponentRegistry.Builder.addChatEngineImageLoaders(koin: Koin) {")
        if (isMatrixEnabled) {
            line("add(MatrixImageFetcher.Factory(koin))", 1)
        }
        line("}")
    }
    writeText(sourceCode)
}

private fun StringBuilder.line(
    text: String = "",
    numTabs: Int = 0,
) {
    for (i in 0 until numTabs) append("    ")
    appendLine(text)
}

private val ChatEngineDependency.function: String get() {
    val function = name.lowercase().uppercaseFirstChar()
    return "get${function}KoinModule"
}