#define MINIMP3_IMPLEMENTATION

#include "minimp3_ex.h"
#include "mpegjni.h"

jlong Java_nand_apps_mpeg_MpegNative_openFileReader(JNIEnv *env, jclass, jstring path) {
    const char *pathPtr = env->GetStringUTFChars(path, NULL);
    mp3dec_ex_t *dec = (mp3dec_ex_t*) malloc(sizeof(mp3dec_ex_t));
    int err = mp3dec_ex_open(dec, pathPtr, MP3D_SEEK_TO_SAMPLE);
    env->ReleaseStringUTFChars(path, pathPtr);
    return err == 0 ? (jlong) dec : NULL;
}

jobject Java_nand_apps_mpeg_MpegNative_getFileInfo(JNIEnv *env, jclass, jlong ptr) {
    mp3dec_ex_t *dec = (mp3dec_ex_t*) ptr;
    mp3dec_frame_info_t info = dec->info;
    jclass cls = env->FindClass("nand/apps/mpeg/MpegFileInfo");
    jmethodID constructor = env->GetMethodID(cls, "<init>", "(JIII)V");
    return env->NewObject(cls, constructor, dec->samples, info.hz, info.channels, info.bitrate_kbps);
}

jlong Java_nand_apps_mpeg_MpegNative_seekTo(JNIEnv *, jclass, jlong ptr, jlong position) {
    return mp3dec_ex_seek((mp3dec_ex_t*)ptr, position) == 0;
}

jint Java_nand_apps_mpeg_MpegNative_readSamples(JNIEnv *env, jclass, jlong ptr, jshortArray buffer) {
    jshort *bufferPtr = env->GetShortArrayElements(buffer, NULL);
    int bufferLength = env->GetArrayLength(buffer);
    int read = mp3dec_ex_read((mp3dec_ex_t*)ptr, bufferPtr, bufferLength);
    env->ReleaseShortArrayElements(buffer, bufferPtr, 0);
    return read;
}

jlong Java_nand_apps_mpeg_MpegNative_getSamplePosition(JNIEnv *, jclass, jlong ptr) {
    mp3dec_ex_t *dec = (mp3dec_ex_t*) ptr;
    return dec->cur_sample;
}

void Java_nand_apps_mpeg_MpegNative_closeReader(JNIEnv *, jclass, jlong ptr) {
    mp3dec_ex_close((mp3dec_ex_t*)ptr);
}