package nand.apps.chat.audio

import nand.apps.chat.audio.id3.Id3TagReader
import nand.apps.chat.io.extension
import nand.apps.chat.model.audio.AudioMetadata
import nand.apps.opus.OpusNative
import okio.Path.Companion.toPath

class AudioMetadataReader(
    private val id3Reader: Id3TagReader,
) {

    fun readMetadata(path: String): AudioMetadata? {
        return when (path.toPath().extension.lowercase()) {
            "opus" -> {
                val filePtr = OpusNative.openFileReader(path)
                if (filePtr != 0L) {
                    AudioMetadata(
                        title = OpusNative.readMetadata(filePtr, "title")?.decodeToString(),
                        artist = OpusNative.readMetadata(filePtr, "artist")?.decodeToString(),
                    )
                } else {
                    null
                }
            }
            "mp3" -> try {
                id3Reader.readTags(path)?.run {
                    AudioMetadata(
                        title = title,
                        artist = artist,
                    )
                }
            } catch (ignore: Exception) {
                null
            }
            else -> null
        }
    }
}