package nand.apps.chat.audio

import nand.apps.audio.AudioFileReader
import nand.apps.audio.AudioFrame
import nand.apps.chat.model.audio.AudioMetadata
import nand.apps.chat.model.settings.notifications.NotificationSoundType
import nand.apps.opus.OpusFileMetadata

const val DEFAULT_AUDIO_SAMPLE_RATE = 48000
const val DEFAULT_AUDIO_CHANNELS = 2

fun AudioPlayer.queueSound(type: NotificationSoundType) =
    queueSound(type.fileName)

fun AudioFileReader.readToFrame(buffer: ShortArray): AudioFrame? {
    val numSamples = read(buffer)
    return when {
        numSamples <= 0 -> null
        else -> AudioFrame(
            samples = when {
                buffer.size == numSamples -> buffer.copyOf()
                else -> buffer.copyOfRange(0, numSamples)
            },
            channels = channels,
            sampleRate = sampleRate,
        )
    }
}

fun AudioMetadata.toOpusMetadata() = OpusFileMetadata(
    title = title,
    artist = artist,
)