package nand.apps.chat.audio

import kotlinx.coroutines.runBlocking
import nand.apps.chat.io.ChatFileSystem
import nand.apps.chat.log.getLogger
import okio.Path
import org.jetbrains.compose.resources.ExperimentalResourceApi
import seers.composeapp.generated.resources.Res

@OptIn(ExperimentalResourceApi::class)
class SoundEffectCache(
    fileSystem: ChatFileSystem,
) {

    private val fileCache = mutableMapOf<String, Path>()
    private val tmpDir = fileSystem.getTempDirectory("sounds")
    private val logger = getLogger("SoundEffectCache")

    fun getSoundEffectPath(name: String): Path? {
        return fileCache[name] ?: runBlocking {
            val fullName = "$name.opus"
            try {
                val bytes = Res.readBytes("files/sounds/$fullName")
                tmpDir.writeToFile(fullName, bytes, safe = false)
                tmpDir.getFilePath(fullName)
            } catch (e: Exception) {
                logger.error("Failed to find sound effect: $fullName")
                null
            }
        }?.also { file ->
            fileCache[name] = file
        }
    }
}