package nand.apps.chat.event

import nand.apps.chat.model.uid.ContactUid
import nand.apps.chat.platform.AppState
import nand.apps.chat.platform.AppStateProvider
import nand.apps.chat.repo.ChatMessageRepo
import nand.apps.chat.ui.navigation.MainRoute
import nand.apps.chat.ui.navigation.NavRoute
import org.koin.core.Koin

class VisibleMessageTracker(koin: Koin) {

    private val messageRepo: ChatMessageRepo by koin.inject()
    private val appStateProvider: AppStateProvider by koin.inject()

    private var conversationUid: ContactUid? = null
    private var currentNavRoute: NavRoute? = null
    var isScrolledUp: Boolean = false
        private set

    fun isConversationVisible(conversationUid: ContactUid) =
        appStateProvider.currentState == AppState.FOREGROUND &&
                currentNavRoute == MainRoute &&
                conversationUid == this.conversationUid

    fun onConversationChanged(conversationUid: ContactUid?) {
        if (this.conversationUid != conversationUid) {
            this.conversationUid = conversationUid
            markConversationRead()
        }
    }

    fun onAppFocused() {
        if (!isScrolledUp) markConversationRead()
    }

    fun onNavRoute(route: NavRoute) {
        if (currentNavRoute != route) {
            currentNavRoute = route
            if (route == MainRoute) markConversationRead()
        }
    }

    fun onScrolledUp(isScrolledUp: Boolean) {
        this.isScrolledUp = isScrolledUp
    }

    fun reset() {
        conversationUid = null
        isScrolledUp = false
    }

    private fun markConversationRead() {
        conversationUid?.let { messageRepo.markConversationRead(it) }
    }
}