package nand.apps.chat.net

import io.ktor.client.*
import io.ktor.client.request.*
import io.ktor.client.statement.*
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.IO
import kotlinx.coroutines.withContext
import nand.apps.chat.io.ChatFileSystem
import nand.apps.chat.io.isFilePath
import nand.apps.chat.log.getLogger
import nand.apps.chat.repo.SettingsRepo
import nand.apps.chat.util.isUrl
import okio.Path.Companion.toPath

class FileDownloaderImpl(
    private val client: HttpClient,
    private val fileSystem: ChatFileSystem,
    private val settingsRepo: SettingsRepo,
) : FileDownloader {

    override val directory get() = settingsRepo.settings.fileSettings.downloadDirectory ?: fileSystem.paths.downloadPath.toString()
    private val logger = getLogger("FileDownloader")
    
    override suspend fun download(
        url: String,
        filePath: String,
    ): Boolean {
        if (!url.isUrl() || !filePath.isFilePath()) return false

        val directory = fileSystem.getDirectoryForFile(filePath) ?: return false
        val path = filePath.toPath()

        return withContext(Dispatchers.IO) {
            try {
                directory.writeToFile(
                    fileName = path.name,
                    data = client.get(url).bodyAsBytes(),
                )
                true
            } catch (e: Exception) {
                logger.error("Error uploading file: $filePath", e)
                false
            }
        }
    }
}