package nand.apps.chat.net.remote

import com.ionspin.kotlin.crypto.secretbox.SecretBox
import com.ionspin.kotlin.crypto.util.LibsodiumRandom
import nand.apps.chat.io.ChatFileEncryption
import nand.apps.chat.model.user.ChatProfileData
import nand.apps.chat.util.sha256Bytes

private const val KEY_BUFFER_SIZE = 32

@OptIn(ExperimentalUnsignedTypes::class)
class ChatRemoteEncryption(
    profile: ChatProfileData,
    val nonce: UByteArray = LibsodiumRandom.buf(NONCE_BYTE_COUNT),
) : ChatFileEncryption {

    private val key = LibsodiumRandom.bufDeterministic(
        size = KEY_BUFFER_SIZE,
        seed = "${profile.username}/${profile.password.orEmpty()}".sha256Bytes().toUByteArray(),
    )

    override fun isEncryptionSupported() = true

    override fun isDecryptionSupported() = true

    override fun isEncrypted(data: ByteArray): Boolean {
        return data.copyOf(NONCE_BYTE_COUNT).contentEquals(nonce.toByteArray())
    }

    override fun encryptData(data: ByteArray): ByteArray? {
        return try {
            val cipher = SecretBox.easy(
                message = data.toUByteArray(),
                nonce = nonce,
                key = key,
            )
            return nonce.toByteArray() + cipher.toByteArray()
        } catch (e: Exception) {
            null
        }
    }

    override fun decryptData(data: ByteArray): ByteArray? {
        return try {
            val cipher = data.copyOfRange(NONCE_BYTE_COUNT, data.size).toUByteArray()
            SecretBox.openEasy(
                ciphertext = cipher,
                nonce = nonce,
                key = key,
            ).toByteArray()
        } catch (e: Exception) {
            null
        }
    }
}