package nand.apps.chat.net.remote.command

import kotlinx.serialization.Serializable
import nand.apps.chat.model.group.PersistedChatGroupInviteData
import nand.apps.chat.model.message.ChatMessageType
import nand.apps.chat.model.user.UserStatus

@Serializable
sealed class ChatRemoteCommand

@Serializable
data class HandshakeCommand(
    val void: String? = null,
) : ChatRemoteCommand()

@Serializable
data class SetNameCommand(
    val name: String,
) : ChatRemoteCommand()

@Serializable
data class SetStatusCommand(
    val status: UserStatus,
) : ChatRemoteCommand()

@Serializable
data class SendMessageCommand(
    val conversationUid: String,
    val contactUid: String,
    val message: String,
    val type: ChatMessageType,
) : ChatRemoteCommand()

/* FRIENDS */

@Serializable
data class SendFriendRequestCommand(
    val friendName: String,
    val friendAddress: String,
    val message: String,
) : ChatRemoteCommand()

@Serializable
data class AcceptFriendRequestCommand(
    val friendUid: String,
) : ChatRemoteCommand()

@Serializable
data class DeclineFriendRequestCommand(
    val friendUid: String,
) : ChatRemoteCommand()

@Serializable
data class RemoveFriendCommand(
    val friendUid: String,
) : ChatRemoteCommand()

/* GROUPS */

@Serializable
data class SendGroupInviteCommand(
    val friendUid: String,
    val groupUid: String,
) : ChatRemoteCommand()

@Serializable
data class AcceptGroupInviteCommand(
    val invite: PersistedChatGroupInviteData,
) : ChatRemoteCommand()

@Serializable
data class DeclineGroupInviteCommand(
    val invite: PersistedChatGroupInviteData,
) : ChatRemoteCommand()