package nand.apps.chat.net.remote.command

import nand.apps.chat.model.group.ChatGroupData
import nand.apps.chat.model.group.toGroupInvite
import nand.apps.chat.model.message.ChatMessage
import nand.apps.chat.model.uid.FriendRequestUid
import nand.apps.chat.model.uid.GroupUid
import nand.apps.chat.model.uid.UserUid
import nand.apps.chat.model.user.FriendRequest
import nand.apps.chat.repo.ChatMessageRepo
import nand.apps.chat.repo.ContactRepo
import nand.apps.chat.repo.SelfProfileRepo

class ChatRemoteCommandHandler(
    private val contactRepo: ContactRepo,
    private val messageRepo: ChatMessageRepo,
    private val selfProfileRepo: SelfProfileRepo,
) {

    fun onCommand(command: ChatRemoteCommand): Boolean {
        return when (command) {
            is HandshakeCommand -> true
            is SetNameCommand -> {
                selfProfileRepo.setCurrentDisplayName(command.name)
                true
            }
            is SetStatusCommand -> {
                selfProfileRepo.setSelfStatus(command.status)
                true
            }
            is SendMessageCommand -> {
                val conversation = contactRepo.getContact(uid = command.conversationUid) ?: return false
                val contact = contactRepo.getContact(
                    uid = command.contactUid,
                    userGroup = conversation as? ChatGroupData,
                ) ?: return false
                messageRepo.sendMessage(
                    ChatMessage(
                        conversationUid = contact.uid,
                        senderUid = contactRepo.self.uid,
                        text = command.message,
                        type = command.type,
                    )
                ) != null
            }
            is SendFriendRequestCommand -> {
                contactRepo.sendFriendRequest(
                    name = command.friendName,
                    address = command.friendAddress,
                    message = command.message,
                ) != null
            }
            is AcceptFriendRequestCommand -> {
                contactRepo.acceptFriendRequest(FriendRequest(FriendRequestUid(command.friendUid))) != null
            }
            is DeclineFriendRequestCommand -> {
                contactRepo.declineFriendRequest(FriendRequest(FriendRequestUid(command.friendUid)))
            }
            is RemoveFriendCommand -> {
                contactRepo.removeFriend(UserUid(command.friendUid))
            }
            is SendGroupInviteCommand -> {
                contactRepo.sendGroupInvite(
                    friendUid = UserUid(command.friendUid),
                    groupUid = GroupUid(command.groupUid),
                )
            }
            is AcceptGroupInviteCommand -> {
                contactRepo.acceptGroupInvite(command.invite.toGroupInvite()) != null
            }
            is DeclineGroupInviteCommand -> {
                contactRepo.removeGroupInvite(command.invite.toGroupInvite())
            }
        }
    }
}