package nand.apps.chat.ui.app

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.*
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import com.hoc081098.kmp.viewmodel.koin.compose.koinKmpViewModel
import nand.apps.chat.model.settings.theme.ChatThemeSetting
import nand.apps.chat.monitor.onUserActivity
import nand.apps.chat.ui.dialog.GenericErrorDialog
import nand.apps.chat.ui.main.MainNavGraph
import nand.apps.chat.ui.main.SplashScreen
import nand.apps.chat.ui.modifier.appendIf
import nand.apps.chat.ui.modifier.onDensityScale
import nand.apps.chat.ui.navigation.LoginRoute
import nand.apps.chat.ui.navigation.MainRoute
import nand.apps.chat.ui.navigation.TutorialRoute
import nand.apps.chat.ui.provider.ChatAppSurface
import nand.apps.chat.ui.provider.LocalLoginState
import nand.apps.chat.ui.snackbar.GlobalSnackbarHost
import nand.apps.chat.ui.update.AppUpdateBalloon

@Composable
fun App(
    viewModel: AppViewModel = koinKmpViewModel(),
) {
    val state by viewModel.stateData.collectAsState()
    
    // Main UI
    ChatAppSurface(
        settings = state.settingsData,
        includeBackground = !state.inTutorial,
        theme = when {
            state.inTutorial -> ChatThemeSetting.DARK
            else -> state.settingsData.themeSettings.theme
        },
        modifier = Modifier
            .appendIf(state.inTutorial) { background(Color.Black) }
            .onUserActivity(viewModel::onUserActivity),
    ) {
        if (state.isAppLoaded) {
            CompositionLocalProvider(
                LocalLoginState provides state.isLoggedIn,
            ) {
                MainNavGraph(
                    startRoute = when {
                        state.inTutorial -> TutorialRoute
                        state.isLoggedIn -> MainRoute
                        else -> LoginRoute
                    },
                    modifier = Modifier
                        .fillMaxSize()
                        .onDensityScale()
                )
            }
            AppUpdateBalloon()
            GlobalSnackbarHost(
                modifier = Modifier
                    .fillMaxWidth()
                    .windowInsetsPadding(WindowInsets.safeDrawing)
                    .align(Alignment.BottomCenter),
            )
            GenericErrorDialog()
        } else {
            SplashScreen()
        }
    }
}