package nand.apps.chat.ui.button

import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.BoxScope
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.padding
import androidx.compose.material.ButtonDefaults
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.draw.drawWithContent
import androidx.compose.ui.geometry.CornerRadius
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.geometry.Size
import androidx.compose.ui.graphics.BlendMode
import androidx.compose.ui.graphics.Brush
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.semantics.Role
import androidx.compose.ui.semantics.role
import androidx.compose.ui.semantics.semantics
import androidx.compose.ui.unit.dp
import nand.apps.chat.ui.modifier.onClick
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.ui.util.toPx

@Composable
fun SimpleButton(
    onClick: () -> Unit,
    modifier: Modifier = Modifier,
    isEnabled: Boolean = true,
    backgroundColor: Color = ChatTheme.colors.primary,
    contentPadding: PaddingValues = ButtonDefaults.ContentPadding,
    content: @Composable BoxScope.() -> Unit,
) {
    val offset = 4.dp.toPx()
    val radius = 4.dp.toPx()
    val innerRadius = CornerRadius(radius, radius)
    val insetSize = offset * 2
    Box(
        contentAlignment = Alignment.Center,
        modifier = modifier
            .semantics { role = Role.Button }
            .clip(ChatTheme.shapes.small)
            .onClick(isEnabled = isEnabled, onClick = onClick)
            .drawWithContent {
                drawRect(
                    color = Color(0xFF4F4F61),
                )
                drawRoundRect(
                    color = Color(0xFF606073),
                    cornerRadius = innerRadius,
                    topLeft = Offset(offset, offset),
                    size = Size(size.width - insetSize, size.height - insetSize),
                )
                if (isEnabled) {
                    drawRect(
                        color = backgroundColor,
                        blendMode = BlendMode.Color,
                    )
                }
                drawRect(
                    brush = Brush.verticalGradient(
                        0.5f to Color.Transparent,
                        1f to Color.Black.copy(0.25f),
                    ),
                )
                drawContent()
            }
            .padding(contentPadding),
        content = content,
    )
}