package nand.apps.chat.ui.button

import androidx.compose.foundation.layout.size
import androidx.compose.material.*
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.TextStyle
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme

@Composable
fun SimpleTextButton(
    text: String,
    onClick: () -> Unit,
    modifier: Modifier = Modifier,
    isEnabled: Boolean = true,
    isLoading: Boolean = false,
    style: TextStyle = ChatTheme.typography.body1,
    backgroundColor: Color = ChatTheme.colors.primary,
    textColor: Color = ChatTheme.colors.onPrimary,
) = SimpleButton(
    isEnabled = isEnabled,
    backgroundColor = backgroundColor,
    onClick = onClick,
    modifier = modifier,
) {
    TextLine(
        text = text,
        style = style,
        color = when {
            isLoading -> Color.Transparent
            isEnabled -> textColor
            else -> textColor.copy(alpha = 0.5f)
        },
        hasShadow = isEnabled,
    )
    if (isLoading) {
        CircularProgressIndicator(
            color = when {
                isEnabled -> textColor
                else -> textColor.copy(alpha = 0.5f)
            },
            modifier = Modifier.size(ChatTheme.dimens.iconSmall),
        )
    }
}