package nand.apps.chat.ui.button

import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.size
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.rotate
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.vector.ImageVector
import androidx.compose.ui.unit.Dp
import nand.apps.chat.ui.icons.StyledIcon
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.StringResource
import org.jetbrains.compose.resources.stringResource

@Composable
fun StyledIconButton(
    icon: ImageVector,
    contentDescription: StringResource,
    onClick: () -> Unit,
    modifier: Modifier = Modifier,
    isEnabled: Boolean = true,
    color: Color = ChatTheme.colors.onPrimary,
    size: Dp = ChatTheme.dimens.iconMedium,
    rotation: Float = 0f,
) = SimpleButton(
    isEnabled = isEnabled,
    onClick = onClick,
    contentPadding = PaddingValues(),
    modifier = modifier,
) {
    StyledIcon(
        icon = icon,
        tint = when {
            isEnabled -> color
            else -> color.copy(alpha = 0.5f)
        },
        contentDescription = stringResource(contentDescription),
        hasShadow = isEnabled,
        modifier = Modifier
            .size(size)
            .rotate(rotation),
    )
}