package nand.apps.chat.ui.chat.message

import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.padding
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalLayoutDirection
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.LayoutDirection
import nand.apps.chat.model.avatar.ChatAvatar
import nand.apps.chat.model.message.ChatMessage
import nand.apps.chat.ui.avatar.ChatAvatarBox
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme

@Composable
fun CompactMessageBlock(
    message: ChatMessage,
    avatar: ChatAvatar,
    displayName: String,
    modifier: Modifier = Modifier,
) = Row(
    modifier = modifier,
) {
    ChatAvatarBox(avatar)
    Column(
        modifier = Modifier.padding(start = ChatTheme.dimens.spacingMedium)
    ) {
        val textAlign = when (LocalLayoutDirection.current) {
            LayoutDirection.Ltr -> TextAlign.Start
            LayoutDirection.Rtl -> TextAlign.End
        }
        MessageAligner {
            TextLine(
                text = displayName,
                style = ChatTheme.typography.caption.copy(fontWeight = FontWeight.Bold),
                color = when {
                    message.isGroupMessage -> ChatTheme.colors.statusOnline
                    else -> ChatTheme.colors.onSurface
                },
                textAlign = textAlign,
            )
        }
        MessageBlock(
            message = message,
            displayName = displayName,
            isCompact = true,
        )
    }
}