package nand.apps.chat.ui.chat.message

import androidx.compose.animation.AnimatedVisibility
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.style.TextAlign
import nand.apps.chat.ui.modifier.onClick
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.pluralStringResource
import seers.composeapp.generated.resources.*

@Composable
fun UnreadMessagesFooter(
    quantity: Int,
    isVisible: Boolean,
    onClick: () -> Unit,
    modifier: Modifier = Modifier,
) = AnimatedVisibility(
    visible = quantity > 0 && isVisible,
    modifier = modifier
        .onClick(onClick = onClick)
        .background(ChatTheme.colors.background)
        .fillMaxWidth(),
) {
    TextLine(
        text = when {
            quantity > 0 -> pluralStringResource(Res.plurals.chat_screen_new_messages_below, quantity, quantity)
            else -> ""
        },
        style = ChatTheme.typography.body2,
        color = ChatTheme.colors.onSurface,
        textAlign = TextAlign.Center,
        modifier = Modifier
            .background(ChatTheme.colors.surface2)
            .padding(ChatTheme.dimens.spacingMedium)
            .fillMaxWidth(),
    )
}