package nand.apps.chat.ui.chat.message.input

import nand.apps.chat.model.contact.ChatContact
import nand.apps.chat.model.group.ChatGroupData
import nand.apps.chat.model.group.ChatGroupRole
import nand.apps.chat.model.group.ChatGroupTalkState
import nand.apps.chat.model.message.ChatMessage
import nand.apps.chat.model.uid.*
import nand.apps.chat.platform.getPlatform
import nand.apps.chat.res.FormattedStringResource
import nand.apps.chat.ui.groups.getLabelResource
import nand.apps.chat.ui.shortcut.ChatKeyEvent
import okio.Path.Companion.toPath
import seers.composeapp.generated.resources.*

data class MessageInputState(
    val isBusy: Boolean = false,
    val isUploadError: Boolean = false,
    val isFileTransferEnabled: Boolean = false,
    val isEmojiPopupVisible: Boolean = false,
    val isGroupUploadDialogVisible: Boolean = false,
    val isSendSelectorVisible: Boolean = false,
    val editMessage: ChatMessage? = null,
    val reactMessage: ChatMessage? = null,
    val conversationUid: ContactUid? = null,
    val conversationContact: ChatContact? = null,
    val conversationTitle: String? = null,
    val talkStateWarning: ChatGroupTalkState? = null,
    val dragFilePath: String? = null,
    val fileConfirmPath: String? = null,
    val mentionIndex: Int = -1,
    val mentionKeyEvent: ChatKeyEvent? = null,
) {

    val conversationGroup = conversationContact as? ChatGroupData
    val isObserver = conversationGroup?.selfRole == ChatGroupRole.OBSERVER
    val isEditable = conversationUid != null && conversationUid !is FriendRequestUid
    val isDirectMessage = conversationUid is UserUid
    val isConversationOnline = conversationContact?.isOnline == true
    val isButtonsEnabled = isEditable && !isBusy && reactMessage == null &&
            !isObserver && talkStateWarning == null && conversationGroup?.error == null
    val isAutoFocus = isButtonsEnabled && getPlatform().isDesktop
    val isMentionPopupVisible = mentionIndex > 0

    val hintText = when {
        isBusy -> {
            FormattedStringResource(Res.string.chat_screen_input_please_wait)
        }
        talkStateWarning != null -> {
            FormattedStringResource(Res.string.group_talk_state_warning, talkStateWarning.getLabelResource())
        }
        isUploadError -> {
            FormattedStringResource(Res.string.chat_screen_input_upload_error)
        }
        !isEditable -> null
        dragFilePath != null -> {
            FormattedStringResource(Res.string.chat_screen_input_drop_file, dragFilePath.toPath().name)
        }
        !isConversationOnline -> {
            FormattedStringResource(Res.string.contact_is_offline, conversationTitle.orEmpty())
        }
        isObserver -> {
            FormattedStringResource(Res.string.group_self_observer_message)
        }
        else -> {
            FormattedStringResource(Res.string.chat_screen_input_type_here)
        }
    }
}
