package nand.apps.chat.ui.chat.screen

import androidx.compose.animation.core.*
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.alpha
import nand.apps.chat.ui.spinner.FollowSpinner
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun ChatScreenNetworkIndicator(
    modifier: Modifier = Modifier,
) = Column(
    horizontalAlignment = Alignment.CenterHorizontally,
    verticalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingSmall),
    modifier = modifier
        .background(
            color = ChatTheme.colors.background,
            shape = RoundedCornerShape(
                bottomStart = ChatTheme.dimens.spacingMedium,
            ),
        )
        .padding(ChatTheme.dimens.spacingMedium),
) {
    val infiniteTransition = rememberInfiniteTransition()
    val alpha by infiniteTransition.animateFloat(
        initialValue = 0.1f,
        targetValue = 1f,
        animationSpec = infiniteRepeatable(tween(1200), RepeatMode.Reverse),
    )
    FollowSpinner()
    TextLine(
        text = stringResource(Res.string.chat_screen_connecting),
        color = ChatTheme.colors.onBackground,
        style = ChatTheme.typography.body2,
        modifier = Modifier.alpha(alpha),
    )
}