package nand.apps.chat.ui.chat.search

import androidx.compose.animation.AnimatedVisibility
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.Close
import androidx.compose.material.icons.outlined.Settings
import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.focus.FocusRequester
import androidx.compose.ui.focus.focusRequester
import nand.apps.chat.ui.button.IconButton
import nand.apps.chat.ui.navigation.BackHandler
import nand.apps.chat.ui.text.SimpleTextField
import nand.apps.chat.ui.text.rememberTextFieldValue
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun ChatSearchHeader(
    state: ChatSearchState,
    onValueChange: (String) -> Unit,
    onCancel: () -> Unit,
    modifier: Modifier = Modifier,
) {
    var terms by rememberTextFieldValue(key = state.isActive)
    var isSettingsVisible by rememberSaveable(state.isActive) { mutableStateOf(false) }
    BackHandler(
        isEnabled = state.isActive,
        onBack = onCancel,
    )
    AnimatedVisibility(
        visible = state.isActive,
        modifier = modifier
            .padding(
                bottom = ChatTheme.dimens.spacingMedium,
                start = ChatTheme.dimens.spacingMedium,
            ),
    ) {
        Row(
            verticalAlignment = Alignment.CenterVertically,
            modifier = Modifier.fillMaxWidth(),
        ) {
            val focusRequester = remember { FocusRequester() }
            LaunchedEffect(state.isActive) {
                if (state.isActive) {
                    focusRequester.requestFocus()
                }
            }
            SimpleTextField(
                labelText = stringResource(Res.string.chat_screen_search_messages),
                value = terms,
                onValueChange = {
                    terms = it
                    onValueChange(it.text)
                },
                trailingIcon = {
                    Row {
                        IconButton(
                            icon = Icons.Outlined.Settings,
                            contentDescription = stringResource(Res.string.chat_screen_search_settings),
                            color = ChatTheme.colors.onBackground2,
                            onClick = { isSettingsVisible = true },
                        )
                        IconButton(
                            icon = Icons.Outlined.Close,
                            contentDescription = stringResource(Res.string.chat_screen_search_cancel),
                            color = ChatTheme.colors.onBackground2,
                            onClick = onCancel,
                        )
                    }
                },
                modifier = Modifier
                    .weight(1f)
                    .focusRequester(focusRequester),
            )
        }
    }
    if (isSettingsVisible && state.isActive) {
        ChatSearchSettingsDialog(
            onDismiss = { isSettingsVisible = false },
        )
    }
}