package nand.apps.chat.ui.chat.search

import androidx.compose.foundation.layout.Column
import androidx.compose.runtime.Composable
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import nand.apps.chat.model.event.UpdateSettingsEvent
import nand.apps.chat.ui.dialog.SimpleDialog
import nand.apps.chat.ui.provider.LocalEventHandler
import nand.apps.chat.ui.provider.LocalSettingsProvider
import nand.apps.chat.ui.text.SimpleTextCheckbox
import seers.composeapp.generated.resources.*

@Composable
fun ChatSearchSettingsDialog(
    onDismiss: () -> Unit,
) {
    val eventHandler = LocalEventHandler.current
    val settings = LocalSettingsProvider.current
    val searchParams = settings.chatSettings.searchParams
    val matchNames = rememberSaveable { mutableStateOf(searchParams.matchNames) }
    val matchCase = rememberSaveable { mutableStateOf(searchParams.matchCase) }
    val matchRegex = rememberSaveable { mutableStateOf(searchParams.matchRegex) }
    val wholeWords = rememberSaveable { mutableStateOf(searchParams.wholeWords) }
    SimpleDialog(
        title = Res.string.chat_screen_search_settings,
        onConfirm = {
            eventHandler += UpdateSettingsEvent(
                settings.copy(
                    chatSettings = settings.chatSettings.copy(
                        searchParams = searchParams.copy(
                            matchNames = matchNames.value,
                            matchCase = matchCase.value,
                            matchRegex = matchRegex.value,
                            wholeWords = wholeWords.value,
                        ),
                    )
                )
            )
            onDismiss()
        },
        onCancel = onDismiss,
    ) {
        Column {
            SimpleTextCheckbox(
                text = Res.string.chat_screen_search_match_names,
                checkState = matchNames,
            )
            SimpleTextCheckbox(
                text = Res.string.chat_screen_search_match_case,
                checkState = matchCase,
            )
            SimpleTextCheckbox(
                text = Res.string.chat_screen_search_match_regex,
                checkState = matchRegex,
                isEnabled = !wholeWords.value,
            )
            SimpleTextCheckbox(
                text = Res.string.chat_screen_search_whole_words,
                checkState = wholeWords,
                isEnabled = !matchRegex.value,
            )
        }
    }
}