package nand.apps.chat.ui.dialog

import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.runtime.*
import androidx.compose.ui.Modifier
import androidx.compose.ui.focus.FocusRequester
import androidx.compose.ui.focus.focusRequester
import nand.apps.chat.ui.text.SimpleTextField
import nand.apps.chat.ui.text.rememberTextFieldValue
import nand.apps.chat.ui.text.takeIfUnderLength
import org.jetbrains.compose.resources.StringResource
import org.jetbrains.compose.resources.stringResource

@Composable
fun EditContactDialog(
    title: StringResource,
    label: StringResource,
    maxLength: Int,
    initialValue: String = "",
    onConfirm: (String) -> Unit,
    onCancel: () -> Unit,
) {
    var value by rememberTextFieldValue(initialValue)
    val focusRequester = remember { FocusRequester() }
    LaunchedEffect(Unit) {
        focusRequester.requestFocus()
    }
    SimpleDialog(
        title = title,
        onConfirm = { onConfirm(value.text) },
        onCancel = onCancel,
        isConfirmEnabled = value.text.isNotBlank(),
    ) {
        SimpleTextField(
            value = value,
            onValueChange = { value = it.takeIfUnderLength(maxLength, value) },
            labelText = stringResource(label),
            onSubmit = { onConfirm(value.text) },
            modifier = Modifier
                .fillMaxWidth()
                .focusRequester(focusRequester),
        )
    }
}