package nand.apps.chat.ui.dialog

import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.RestartAlt
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.focus.FocusRequester
import androidx.compose.ui.focus.focusRequester
import androidx.compose.ui.text.TextRange
import androidx.compose.ui.text.input.TextFieldValue
import nand.apps.chat.model.event.SetFriendAliasEvent
import nand.apps.chat.model.user.UserData
import nand.apps.chat.ui.provider.LocalChatEngineConstraints
import nand.apps.chat.ui.provider.LocalEventHandler
import nand.apps.chat.ui.text.SimpleTextField
import nand.apps.chat.ui.text.SimpleTextFieldButton
import nand.apps.chat.ui.text.rememberTextFieldValue
import nand.apps.chat.ui.text.takeIfUnderLength
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun FriendAliasDialog(
    friend: UserData,
    onDismiss: () -> Unit,
) {
    val constraints = LocalChatEngineConstraints.current
    val eventHandler = LocalEventHandler.current
    var alias by rememberTextFieldValue(friend.alias ?: friend.displayName)
    val focusRequester = remember { FocusRequester() }
    val onSubmit = {
        eventHandler += SetFriendAliasEvent(friend.uid, alias.text.takeIf { it != friend.name })
        onDismiss()
    }
    LaunchedEffect(Unit) {
        focusRequester.requestFocus()
    }
    SimpleDialog(
        title = Res.string.set_alias,
        onConfirm = onSubmit,
        onCancel = onDismiss,
        isConfirmEnabled = alias.text.isNotBlank(),
    ) {
        Row(
            verticalAlignment = Alignment.CenterVertically,
            horizontalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingMedium),
            modifier = Modifier.fillMaxWidth(),
        ) {
            SimpleTextField(
                value = alias,
                onValueChange = { alias = it.takeIfUnderLength(constraints.maxNameLength, alias) },
                labelText = stringResource(Res.string.alias),
                onSubmit = { onSubmit() },
                modifier = Modifier
                    .weight(1f)
                    .focusRequester(focusRequester),
            )
            SimpleTextFieldButton(
                buttonIcon = Icons.Outlined.RestartAlt,
                buttonDescription = Res.string.reset,
                onClick = {
                    val defaultName = friend.name.ifBlank { friend.uid.toString() }
                    alias = TextFieldValue(
                        text = defaultName,
                        selection = TextRange(defaultName.length),
                    )
                },
                modifier = Modifier.padding(
                    top = ChatTheme.dimens.spacingMedium,
                ),
            )
        }
    }
}