package nand.apps.chat.ui.embed

import androidx.compose.runtime.*
import androidx.compose.ui.Modifier
import nand.apps.chat.embed.EmbedPreviewFactory
import nand.apps.chat.embed.amazon.AmazonProductMetadata
import nand.apps.chat.embed.ebay.EbayListingMetadata
import nand.apps.chat.embed.file.AudioEmbedData
import nand.apps.chat.embed.file.ImageEmbedData
import nand.apps.chat.embed.fourchan.FourChanPostMetadata
import nand.apps.chat.embed.mediawiki.MediaWikiPageMetadata
import nand.apps.chat.embed.stackexchange.StackExchangePageMetadata
import nand.apps.chat.embed.web.WebPageEmbedData
import nand.apps.chat.embed.youtube.channel.YoutubeChannelMetadata
import nand.apps.chat.embed.youtube.video.YoutubeVideoMetadata
import nand.apps.chat.model.event.*
import nand.apps.chat.model.message.ChatMessage
import nand.apps.chat.ui.embed.amazon.Renderer
import nand.apps.chat.ui.embed.audio.Renderer
import nand.apps.chat.ui.embed.ebay.Renderer
import nand.apps.chat.ui.embed.fourchan.Renderer
import nand.apps.chat.ui.embed.image.Renderer
import nand.apps.chat.ui.embed.mediawiki.Renderer
import nand.apps.chat.ui.embed.stackexchange.Renderer
import nand.apps.chat.ui.embed.web.Renderer
import nand.apps.chat.ui.embed.youtube.Renderer
import nand.apps.chat.ui.provider.LocalEventHandler
import nand.apps.chat.ui.provider.LocalSettingsProvider
import org.koin.compose.getKoin

@Composable
fun EmbedContainer(
    url: String,
    modifier: Modifier = Modifier,
    message: ChatMessage? = null,
) {
    val factory: EmbedPreviewFactory by getKoin().inject()
    val settings = LocalSettingsProvider.current.embedSettings
    if (!settings.isEnabled || !factory.isHostAllowed(url)) return
    
    val eventHandler = LocalEventHandler.current
    
    var embedData by remember(url, message?.embedData) {
        mutableStateOf(message?.embedData ?: factory.getCachedEmbedData(url))
    }
    when {
        embedData == null -> {
            LaunchedEffect(url) {
                factory.getEmbedData(url)?.let { newEmbedData ->
                    if (message != null) {
                        eventHandler += UpdateMessageEvent(message.copy(embedData = newEmbedData))
                    } else {
                        embedData = newEmbedData
                    }
                }
            }
        }
        message != null && embedData != message.embedData -> {
            eventHandler += UpdateMessageEvent(message.copy(embedData = embedData))
            return
        }
    }
    embedData?.let { embed ->
        if (embed is ImageEmbedData && !settings.isImagesEnabled) return
        when (embed) {
            is AmazonProductMetadata -> embed.Renderer(modifier)
            is AudioEmbedData -> embed.Renderer(modifier)
            is EbayListingMetadata -> embed.Renderer(modifier)
            is FourChanPostMetadata -> embed.Renderer(modifier)
            is ImageEmbedData -> embed.Renderer(modifier)
            is MediaWikiPageMetadata -> embed.Renderer(modifier)
            is StackExchangePageMetadata -> embed.Renderer(modifier)
            is WebPageEmbedData -> embed.Renderer(modifier)
            is YoutubeChannelMetadata -> embed.Renderer(modifier)
            is YoutubeVideoMetadata -> embed.Renderer(modifier)
        }
    }
}
