package nand.apps.chat.ui.embed.image

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Download
import androidx.compose.material.icons.filled.OpenInBrowser
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalUriHandler
import nand.apps.chat.io.isFilePath
import nand.apps.chat.net.openFile
import nand.apps.chat.res.getStringBlocking
import nand.apps.chat.ui.dialog.FileDownloadDialog
import nand.apps.chat.ui.menu.ActionMenuArea
import nand.apps.chat.ui.menu.ActionMenuItem
import nand.apps.chat.util.isUrl
import seers.composeapp.generated.resources.*

@Composable
fun ImageEmbedActionMenu(
    model: Any,
    modifier: Modifier = Modifier,
    onClick: (() -> Unit)? = null,
    content: @Composable () -> Unit,
) {
    val uriHandler = LocalUriHandler.current
    var isDownloadPromptVisible by rememberSaveable { mutableStateOf(false) }
    ActionMenuArea(
        isEnabled = model is String,
        onClick = onClick,
        items = {
            val uri  = model as? String ?: return@ActionMenuArea emptyList()
            when {
                uri.isUrl() -> listOf(
                    ActionMenuItem(
                        icon = Icons.Default.Download,
                        label = getStringBlocking(Res.string.download),
                        onClick = { isDownloadPromptVisible = true },
                    ),
                    ActionMenuItem(
                        icon = Icons.Default.OpenInBrowser,
                        label = getStringBlocking(Res.string.fullscreen_preview_open_url),
                        onClick = { uriHandler.openUri(uri) },
                    ),
                )
                uri.isFilePath() -> listOf(
                    ActionMenuItem(
                        icon = Icons.Default.OpenInBrowser,
                        label = getStringBlocking(Res.string.fullscreen_preview_open_file),
                        onClick = { uriHandler.openFile(uri) },
                    ),
                )
                else -> emptyList()
            }
        },
        content = content,
        modifier = modifier,
    )
    if (isDownloadPromptVisible && model is String && model.isUrl()) {
        FileDownloadDialog(
            url = model,
            onConfirm = { isDownloadPromptVisible = false },
            onCancel = { isDownloadPromptVisible = false },
        )
    }
}