package nand.apps.chat.ui.embed.youtube

import androidx.compose.foundation.layout.*
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalUriHandler
import androidx.compose.ui.text.style.TextOverflow
import nand.apps.chat.embed.youtube.channel.YoutubeChannelMetadata
import nand.apps.chat.ui.embed.EmbedThumbnailRow
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.util.takeIfNotBlank

@Composable
fun YoutubeChannelMetadata.Renderer(
    modifier: Modifier = Modifier,
) {
    val uriHandler = LocalUriHandler.current
    EmbedThumbnailRow(
        thumbnailUrl = thumbnailUrl,
        thumbnailShape = CircleShape,
        contentDescription = name,
        modifier = modifier,
        clickable = url != null,
        onClick = { url?.let { uriHandler.openUri(it) } },
    ) {
        TextLine(
            text = name.orEmpty(),
            style = ChatTheme.typography.h5,
        )
        listOfNotNull(handle, subscriberCount, videoCount).joinToString(" • ").takeIfNotBlank()?.let {
            TextLine(
                text = it,
                style = ChatTheme.typography.body1,
            )
        }
        Text(
            text = description.orEmpty(),
            color = ChatTheme.colors.onBackground,
            style = ChatTheme.typography.body2,
            maxLines = 3,
            overflow = TextOverflow.Ellipsis,
            modifier = Modifier.padding(top = ChatTheme.dimens.spacingSmall)
        )
    }
}