package nand.apps.chat.ui.emoji

import com.hoc081098.kmp.viewmodel.ViewModel
import kotlinx.coroutines.Job
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.launch
import nand.apps.chat.emoji.EmojiRepo
import nand.apps.chat.model.emoji.EmojiGroup
import nand.apps.chat.repo.SettingsRepo
import nand.apps.chat.util.fifoSharedFlow

class EmojiPopupViewModel(
    private val repo: EmojiRepo,
    private val settingsRepo: SettingsRepo,
) : ViewModel() {

    val stateData = settingsRepo.settings.chatSettings.run {
        MutableStateFlow(
            EmojiPopupState(
                currentGroup = emojiPopupGroup,
                isSearchVisible = isEmojiPopupSearchEnabled,
            )
        )
    }
    val actionData = fifoSharedFlow<EmojiPopupAction>()
    private val state get() = stateData.value

    private var refreshJob: Job? = null

    fun refresh() {
        refreshJob?.cancel()
        refreshJob = viewModelScope.launch {
            val results = repo.getEmojis(state.currentGroup, state.searchTerms)
            updateState {
                it.copy(
                    emojis = results,
                    groups = it.groups.ifEmpty { repo.groups.keys.toList() },
                    isLoading = false,
                )
            }
        }
    }

    fun setGroup(group: EmojiGroup) {
        updateState {
            settingsRepo.update { chatSettings.copy(emojiPopupGroup = group) }
            it.copy(currentGroup = group)
        }
        actionData.tryEmit(ClearSearchAction(focusInput = true))
        refresh()
    }

    fun toggleSearch() {
        updateState {
            val isSearchVisible = !it.isSearchVisible
            settingsRepo.update { chatSettings.copy(isEmojiPopupSearchEnabled = isSearchVisible) }
            it.copy(
                isSearchVisible = isSearchVisible,
                searchTerms = "",
            )
        }
        actionData.tryEmit(ClearSearchAction())
        refresh()
    }

    fun search(terms: String) {
        updateState { it.copy(searchTerms = terms) }
        refresh()
    }

    private fun updateState(block: (EmojiPopupState) -> EmojiPopupState) {
        stateData.tryEmit(block(state))
    }
}