package nand.apps.chat.ui.file

import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.width
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp
import nand.apps.chat.io.ChatFileSortType
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.StringResource
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun ChatFileSortOptionsRow(
    sortType: ChatFileSortType,
    fileSizeWidth: Dp,
    fileDateWidth: Dp,
    isEnabled: Boolean,
    onSelected: (ChatFileSortType) -> Unit,
    modifier: Modifier = Modifier,
    optionModifier: Modifier = Modifier,
    textStyle: TextStyle = ChatTheme.typography.body2,
) = Row(
    modifier = modifier,
) {
    OptionSection(
        label = Res.string.file_name,
        textStyle = textStyle,
        textAlign = TextAlign.Start,
        isSelected = sortType == ChatFileSortType.NAME,
        isEnabled = isEnabled,
        onClick = { onSelected(ChatFileSortType.NAME) },
        modifier = optionModifier.weight(1f),
    )
    OptionSection(
        label = Res.string.file_size,
        textStyle = textStyle,
        textAlign = TextAlign.End,
        isSelected = sortType == ChatFileSortType.SIZE,
        isEnabled = isEnabled,
        onClick = { onSelected(ChatFileSortType.SIZE) },
        modifier = optionModifier.width(fileSizeWidth),
    )
    if (fileDateWidth > 0.dp) {
        OptionSection(
            label = Res.string.file_date,
            textStyle = textStyle,
            textAlign = TextAlign.End,
            isSelected = sortType == ChatFileSortType.DATE,
            isEnabled = isEnabled,
            onClick = { onSelected(ChatFileSortType.DATE) },
            modifier = optionModifier.width(fileDateWidth),
        )
    }
}

@Composable
private fun OptionSection(
    label: StringResource,
    textStyle: TextStyle,
    textAlign: TextAlign,
    isSelected: Boolean,
    isEnabled: Boolean,
    onClick: () -> Unit,
    modifier: Modifier,
) {
    TextLine(
        text = stringResource(label),
        style = textStyle,
        color = if (isEnabled) ChatTheme.colors.onSurface else ChatTheme.colors.onBackground2,
        textAlign = textAlign,
        modifier = Modifier
            .background(if (isSelected) ChatTheme.colors.surface3 else Color.Unspecified)
            .clickable(enabled = isEnabled, onClick = onClick)
            .then(modifier),
    )
}