package nand.apps.chat.ui.groups.settings

import com.hoc081098.kmp.viewmodel.ViewModel
import nand.apps.chat.model.group.ChatGroupData
import nand.apps.chat.model.group.ChatGroupFeature
import nand.apps.chat.model.group.ChatGroupType
import nand.apps.chat.repo.ContactRepo
import nand.apps.chat.repo.SelfProfileRepo

class GroupSettingsDialogViewModel(
    private val contactRepo: ContactRepo,
    private val selfProfileRepo: SelfProfileRepo,
) : ViewModel() {

    fun update(group: ChatGroupData) {
        val selfPeer = group.findSelf(selfProfileRepo.self)
        if (group.isTitleEditable) {
            contactRepo.setGroupTitle(group.uid, group.title)
        }
        if (group.isDescEditable && (selfPeer.isStaffRole || !group.isDescriptionLocked)) {
            contactRepo.setGroupDescription(group.uid, group.description)
            if (selfPeer.isStaffRole) {
                contactRepo.setGroupDescriptionLocked(group.uid, group.isDescriptionLocked)
            }
        }
        if (selfPeer.isFounder) {
            if (ChatGroupFeature.PASSWORD in group.features) {
                contactRepo.setGroupPassword(group.uid, group.password)
            }
            if (ChatGroupFeature.PEER_LIMIT in group.features) {
                contactRepo.setGroupPeerLimit(group.uid, group.peerLimit)
            }
            if (group.hasRoles) {
                contactRepo.setGroupTalkState(group.uid, group.talkState)
            }
            if (group.type == ChatGroupType.ADVANCED) {
                contactRepo.setGroupVisibility(group.uid, group.visibility)
            }
        }
    }
}