package nand.apps.chat.ui.logo

import androidx.compose.animation.animateColor
import androidx.compose.animation.core.*
import androidx.compose.foundation.Image
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.size
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.alpha
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.ColorFilter
import androidx.compose.ui.unit.DpSize
import androidx.compose.ui.unit.dp
import nand.apps.chat.BuildConfig.APP_NAME
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.painterResource
import seers.composeapp.generated.resources.*

private val LOGO_COLOR_START = Color(0xFFffe57f)
private val LOGO_COLOR_END = Color(0xFFfff6d4)
private val LOGO_TEXT_SIZE = DpSize(298.dp, 90.dp)

@Composable
fun SeersLogoText(
    modifier: Modifier = Modifier,
) = Box(
    contentAlignment = Alignment.Center,
    modifier = modifier.size(LOGO_TEXT_SIZE),
) {
    if (ChatTheme.colors.isLight) {
        val color = ChatTheme.colors.onBackground2
        val colorFilter = remember(color) { ColorFilter.tint(color) }
        Image(
            painter = painterResource(Res.drawable.seers_logo_text),
            contentDescription = APP_NAME,
            colorFilter = colorFilter,
        )
    } else {
        val infiniteTransition = rememberInfiniteTransition()
        val baseColor by infiniteTransition.animateColor(
            initialValue = LOGO_COLOR_START,
            targetValue = LOGO_COLOR_END,
            animationSpec = infiniteRepeatable(
                animation = tween(2000, easing = FastOutSlowInEasing),
                repeatMode = RepeatMode.Reverse,
            )
        )
        val glowAmount by infiniteTransition.animateFloat(
            initialValue = 0.25f,
            targetValue = 1f,
            animationSpec = infiniteRepeatable(
                animation = tween(2000, easing = FastOutSlowInEasing),
                repeatMode = RepeatMode.Reverse,
            ),
            label = "Logo Glow",
        )
        Image(
            painter = painterResource(Res.drawable.seers_logo_text_glow),
            contentDescription = null,
            colorFilter = remember(LOGO_COLOR) { ColorFilter.tint(LOGO_COLOR) },
            modifier = Modifier
                .matchParentSize()
                .alpha(glowAmount),
        )
        Image(
            painter = painterResource(Res.drawable.seers_logo_text),
            contentDescription = APP_NAME,
            colorFilter = remember(baseColor) { ColorFilter.tint(baseColor) },
        )
    }
}