package nand.apps.chat.ui.modifier

import androidx.compose.foundation.clickable
import androidx.compose.foundation.gestures.awaitEachGesture
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.Modifier
import androidx.compose.ui.composed
import androidx.compose.ui.draw.alpha
import androidx.compose.ui.draw.blur
import androidx.compose.ui.input.pointer.*
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp
import nand.apps.chat.util.fastForEach
import androidx.compose.foundation.ScrollState
import androidx.compose.foundation.layout.*
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.setValue
import androidx.compose.ui.focus.onFocusEvent
import androidx.compose.ui.layout.onGloballyPositioned
import androidx.compose.ui.layout.positionInRoot
import kotlinx.coroutines.launch

inline fun Modifier.appendIf(condition: Boolean, block: Modifier.() -> Modifier) =
    if (condition) block() else this

fun Modifier.onClick(
    isEnabled: Boolean = true,
    onClick: () -> Unit,
) = clickable(
    enabled = isEnabled,
    onClick = onClick,
).appendIf(isEnabled) {
    pointerHoverIcon(PointerIcon.Hand)
}

fun Modifier.onScrollWheel(
    block: (Float, PointerKeyboardModifiers) -> Unit,
): Modifier {
    return pointerInput(Unit) {
        awaitEachGesture {
            val event = awaitPointerEvent()
            if (event.type == PointerEventType.Scroll) {
                event.changes.fastForEach { block(it.scrollDelta.y, event.keyboardModifiers) }
            }
        }
    }
}

fun Modifier.blockInput() = pointerInput(Unit) {}

fun Modifier.bringIntoView(scrollState: ScrollState): Modifier = composed {
    var scrollToPosition by remember { mutableStateOf(0f) }
    val coroutineScope = rememberCoroutineScope()
    onGloballyPositioned { coordinates ->
        scrollToPosition = scrollState.value + coordinates.positionInRoot().y
    }.onFocusEvent {
        if (it.isFocused) {
            coroutineScope.launch {
                scrollState.animateScrollTo(scrollToPosition.toInt())
            }
        }
    }
}

fun Modifier.shadow(
    offsetX: Dp = 0.dp,
    offsetY: Dp = 1.dp,
    blur: Dp = 1.dp,
    alpha: Float = 0.75f,
) = offset(offsetX, offsetY)
    .alpha(alpha)
    .blur(blur)

fun Modifier.windowInsetPaddingStart(insets: WindowInsets) =
    windowInsetsPadding(insets.only(WindowInsetsSides.Start))