package nand.apps.chat.ui.settings

import androidx.compose.foundation.layout.*
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.Dp
import nand.apps.chat.ui.modifier.appendIf
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.ui.toolbar.SimpleNavToolbar
import org.jetbrains.compose.resources.StringResource

@Composable
fun SettingsColumn(
    title: StringResource?,
    modifier: Modifier = Modifier,
    isScrollable: Boolean = true,
    contentPadding: Dp = ChatTheme.dimens.spacingLarge,
    endContent: @Composable RowScope.() -> Unit = {},
    content: @Composable ColumnScope.() -> Unit,
) = Column(modifier = modifier) {
    SimpleNavToolbar(
        text = title,
        endContent = endContent,
    )
    Column(
        modifier = Modifier
            .fillMaxSize()
            .padding(horizontal = contentPadding)
            .appendIf(isScrollable) { verticalScroll(rememberScrollState()) }
            .padding(vertical = contentPadding),
        content = content,
    )
}