package nand.apps.chat.ui.settings

import com.hoc081098.kmp.viewmodel.ViewModel
import kotlinx.coroutines.flow.MutableStateFlow
import nand.apps.chat.model.settings.*
import nand.apps.chat.model.settings.call.CallSettingsData
import nand.apps.chat.model.settings.filetransfer.FileTransferSettingsData
import nand.apps.chat.model.settings.net.NetworkSettingsData
import nand.apps.chat.model.settings.notifications.ChatTraySettingsData
import nand.apps.chat.model.settings.notifications.ChatWidgetSettingsData
import nand.apps.chat.model.settings.notifications.NotificationSettingsData
import nand.apps.chat.model.settings.notifications.NotificationSoundSettings
import nand.apps.chat.model.settings.shortcut.ShortcutAction
import nand.apps.chat.model.settings.shortcut.ShortcutKeyCombo
import nand.apps.chat.model.settings.theme.ThemeSettingsData
import nand.apps.chat.model.settings.update.AppUpdateSettingsData
import nand.apps.chat.repo.SettingsRepo
import nand.apps.chat.ui.util.collect

class SettingsViewModel(
    private val repo: SettingsRepo,
) : ViewModel() {

    private val stateData = MutableStateFlow(repo.settings)
    
    init {
        collect(repo.observer) { stateData.tryEmit(it) }
    }
    
    fun update(data: GeneralSettingsData) =
        update(stateData.value.copy(generalSettings = data))
    
    fun update(data: ChatSettingsData) =
        update(stateData.value.copy(chatSettings = data))

    fun update(data: CallSettingsData) =
        update(stateData.value.copy(callSettings = data))

    fun update(data: NotificationSettingsData) =
        update(stateData.value.copy(notificationSettings = data))

    fun update(data: EmbedSettingsData) =
        update(stateData.value.copy(embedSettings = data))

    fun update(data: FileTransferSettingsData) =
        update(stateData.value.copy(fileSettings = data))

    fun update(data: NotificationSoundSettings) =
        update(stateData.value.notificationSettings.copy(soundSettings = data))

    fun update(data: ChatWidgetSettingsData) =
        update(stateData.value.notificationSettings.copy(widgetSettings = data))

    fun update(data: ChatTraySettingsData) =
        update(stateData.value.notificationSettings.copy(traySettings = data))

    fun update(data: NetworkSettingsData) =
        update(stateData.value.copy(networkSettings = data))

    fun update(data: ThemeSettingsData) =
        update(stateData.value.copy(themeSettings = data))

    fun update(data: AppUpdateSettingsData) =
        update(stateData.value.copy(appUpdateSettings = data))

    fun update(data: DevSettingsData) =
        update(stateData.value.copy(devSettings = data))
    
    fun setShortcutKey(action: ShortcutAction, combo: ShortcutKeyCombo?) {
        val settings = stateData.value.shortcutSettings
        val shortcuts = settings.shortcuts.toMutableMap()
        combo?.let { shortcuts[action] = it.toString() } ?: shortcuts.remove(action)
        update(stateData.value.copy(shortcutSettings = settings.copy(shortcuts = shortcuts)))
    }
    
    private fun update(data: SettingsData) {
        repo.settings = data
    }
}