package nand.apps.chat.ui.settings.call

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.*
import androidx.compose.runtime.Composable
import nand.apps.chat.ui.provider.LocalSettingsProvider
import nand.apps.chat.ui.settings.SettingsViewModel
import nand.apps.chat.ui.settings.item.SettingsItem
import nand.apps.chat.ui.settings.row.InputDialogRow
import nand.apps.chat.util.REGEX_NON_DIGITS
import nand.apps.chat.util.toIntOrZero
import seers.composeapp.generated.resources.*

class CallMaxRingsSettingsItem : SettingsItem {

    override val title = Res.string.settings_calls_max_rings_title
    override val description = Res.string.settings_calls_max_rings_desc
    override val icon = Icons.Outlined.PhoneInTalk

    @Composable
    override fun GetComposable(viewModel: SettingsViewModel) {
        val state = LocalSettingsProvider.current.callSettings
        InputDialogRow(
            label = Res.string.settings_calls_max_rings_label,
            value = state.maxRings.toString(),
            regex = REGEX_NON_DIGITS,
            isEnabled = state.isEnabled,
            onConfirm = { rings ->
                viewModel.update(state.copy(maxRings = rings.toIntOrZero().coerceAtLeast(1)))
            }
        )
    }
}