package nand.apps.chat.ui.settings.call

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.*
import androidx.compose.runtime.Composable
import nand.apps.chat.ui.provider.LocalSettingsProvider
import nand.apps.chat.ui.settings.SettingsViewModel
import nand.apps.chat.ui.settings.item.SettingsItem
import nand.apps.chat.ui.settings.row.InputDialogRow
import nand.apps.chat.util.REGEX_NON_DIGITS
import nand.apps.chat.util.clamp
import nand.apps.chat.util.toIntOrZero
import seers.composeapp.generated.resources.*

class MaxVoicemailDurationSettingsItem : SettingsItem {

    override val title = Res.string.settings_calls_max_voicemail_duration_title
    override val description = Res.string.settings_calls_max_voicemail_duration_desc
    override val icon = Icons.Outlined.HourglassEmpty

    @Composable
    override fun GetComposable(viewModel: SettingsViewModel) {
        val state = LocalSettingsProvider.current.callSettings
        InputDialogRow(
            label = Res.string.settings_calls_max_voicemail_duration_label,
            value = (state.voicemailMaxDuration / 1000).toString(),
            regex = REGEX_NON_DIGITS,
            isEnabled = state.isVoicemailEnabled,
            onConfirm = { seconds ->
                viewModel.update(state.copy(voicemailMaxDuration = seconds.toIntOrZero().clamp(5, 600) * 1000))
            }
        )
    }
}