package nand.apps.chat.ui.settings.call

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.*
import androidx.compose.runtime.Composable
import nand.apps.chat.ui.provider.LocalSettingsProvider
import nand.apps.chat.ui.settings.SettingsViewModel
import nand.apps.chat.ui.settings.dialog.BitRateSettingDialog
import nand.apps.chat.ui.settings.item.SettingsItem
import nand.apps.chat.ui.settings.row.DialogRow
import seers.composeapp.generated.resources.*

private val AUDIO_BIT_RATE_OPTIONS = listOf(16, 32, 64)

class VoicemailBitRateSettingsItem : SettingsItem {

    override val title = Res.string.settings_calls_voicemail_bit_rate_title
    override val description = Res.string.settings_calls_voicemail_bit_rate_desc
    override val icon = Icons.Outlined.GraphicEq

    @Composable
    override fun GetComposable(viewModel: SettingsViewModel) {
        val state = LocalSettingsProvider.current.callSettings
        DialogRow(
            isEnabled = state.isVoicemailEnabled,
        ) { onConfirm ->
            BitRateSettingDialog(
                title = Res.string.settings_calls_voicemail_bit_rate_title,
                kbpsOptions = AUDIO_BIT_RATE_OPTIONS,
                defaultKbps = state.voicemailAudioBitRate / 1000,
                onConfirm = { kbps ->
                    viewModel.update(state.copy(voicemailAudioBitRate = kbps * 1000))
                    onConfirm()
                },
                onCancel = onConfirm,
            )
        }
    }
}