package nand.apps.chat.ui.settings.call

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.*
import androidx.compose.runtime.Composable
import nand.apps.chat.model.audio.AudioMetadata
import nand.apps.chat.repo.VoicemailRepo
import nand.apps.chat.ui.dialog.audio.AudioRecorderDialog
import nand.apps.chat.ui.provider.LocalSettingsProvider
import nand.apps.chat.ui.settings.SettingsViewModel
import nand.apps.chat.ui.settings.item.SettingsItem
import nand.apps.chat.ui.settings.row.DialogRow
import nand.apps.chat.res.getStringBlocking
import org.koin.compose.getKoin
import seers.composeapp.generated.resources.*

class VoicemailGreetingSettingsItem : SettingsItem {

    override val title = Res.string.settings_calls_voicemail_greeting_title
    override val description = Res.string.settings_calls_voicemail_greeting_desc
    override val icon = Icons.Outlined.WavingHand

    @Composable
    override fun GetComposable(viewModel: SettingsViewModel) {
        val state = LocalSettingsProvider.current.callSettings
        DialogRow(
            isEnabled = state.isVoicemailEnabled,
            isLoginRequired = true,
        ) { onConfirm ->
            val voicemailRepo: VoicemailRepo by getKoin().inject()
            AudioRecorderDialog(
                title = Res.string.settings_calls_voicemail_greeting_title,
                filePath = voicemailRepo.getGreetingAudioPath().toString(),
                recordBitRate = state.voicemailAudioBitRate,
                metadata = AudioMetadata(
                    title = getStringBlocking(Res.string.settings_calls_voicemail_greeting_title),
                ),
                shouldEncrypt = true,
                onConfirm = onConfirm,
                onDismiss = onConfirm,
            )
        }
    }
}