package nand.apps.chat.ui.settings.chat

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.*
import androidx.compose.runtime.Composable
import nand.apps.chat.ui.provider.LocalSettingsProvider
import nand.apps.chat.ui.settings.SettingsViewModel
import nand.apps.chat.ui.settings.item.*
import nand.apps.chat.ui.settings.row.InputDialogRow
import nand.apps.chat.util.REGEX_NON_DIGITS
import nand.apps.chat.util.toIntOrZero
import seers.composeapp.generated.resources.*

class ChatMaxMessagesSettingsItem : SettingsItem {

    override val title = Res.string.settings_chat_max_messages_title
    override val description = Res.string.settings_chat_max_messages_desc
    override val icon = Icons.Outlined.FormatListNumbered

    @Composable
    override fun GetComposable(viewModel: SettingsViewModel) {
        val state = LocalSettingsProvider.current.chatSettings
        InputDialogRow(
            label = Res.string.settings_chat_max_messages_label,
            value = state.maxMessages.toString(),
            regex = REGEX_NON_DIGITS,
            onConfirm = { maxMessages ->
                viewModel.update(state.copy(maxMessages = maxMessages.toIntOrZero().coerceAtLeast(50)))
            }
        )
    }
}