package nand.apps.chat.ui.settings.dev

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.*
import androidx.compose.runtime.Composable
import nand.apps.chat.model.settings.shortcut.ShortcutAction
import nand.apps.chat.ui.provider.LocalSettingsProvider
import nand.apps.chat.ui.settings.SettingsViewModel
import nand.apps.chat.ui.settings.item.*
import nand.apps.chat.ui.settings.row.CheckboxRow
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import seers.composeapp.generated.resources.*

class DevConsoleEnabledSettingsItem : SettingsItem {

    override val title = Res.string.settings_dev_console_enabled_title
    override val description = Res.string.settings_dev_console_enabled_desc
    override val icon = Icons.Outlined.LogoDev

    @Composable
    override fun GetComposable(viewModel: SettingsViewModel) {
        val settings = LocalSettingsProvider.current
        val state = settings.devSettings
        val devConsoleShortcut = settings.shortcutSettings.shortcuts[ShortcutAction.TOGGLE_DEV_CONSOLE]
        CheckboxRow(
            isChecked = state.isDevConsoleEnabled,
            onCheckedChange = { checked ->
                viewModel.update(state.copy(isDevConsoleEnabled = checked))
            },
            endContent = {
                devConsoleShortcut?.let {
                    TextLine(
                        text = it,
                        style = ChatTheme.typography.caption,
                        color = ChatTheme.colors.onBackground2,
                    )
                }
            }
        )
    }
}