package nand.apps.chat.ui.settings.dialog

import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.focus.FocusDirection
import androidx.compose.ui.platform.LocalFocusManager
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardType
import nand.apps.chat.model.settings.net.ChatRemoteSettingsData
import nand.apps.chat.ui.dialog.SimpleDialog
import nand.apps.chat.ui.provider.LocalSettingsProvider
import nand.apps.chat.ui.text.SimpleTextField
import nand.apps.chat.ui.text.rememberTextFieldValue
import nand.apps.chat.ui.text.takeIfNumber
import nand.apps.chat.ui.text.takeIfValidHost
import nand.apps.chat.util.takeIfNotBlank
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun NetworkRemoteConfigDialog(
    onUpdate: (ChatRemoteSettingsData) -> Unit,
    onDismiss: () -> Unit,
) {
    val focusManager = LocalFocusManager.current
    val networkSettings = LocalSettingsProvider.current.networkSettings
    val config = networkSettings.remoteSettings
    var host by rememberTextFieldValue(config.host)
    var port by rememberTextFieldValue(config.port.toString())
    SimpleDialog(
        title = Res.string.settings_network_remote_config_title,
        isConfirmEnabled = host.text.isNotBlank() && port.text.toIntOrNull() != null,
        onConfirm = {
            onUpdate(
                config.copy(
                    host = host.text.takeIfNotBlank() ?: config.host,
                    port = port.text.toIntOrNull() ?: config.port,
                )
            )
        },
        onCancel = onDismiss,
    ) {
        Column {
            SimpleTextField(
                value = host,
                onValueChange = { host = it.takeIfValidHost(host) },
                labelText = stringResource(Res.string.settings_network_host_label),
                keyboardOptions = KeyboardOptions(
                    autoCorrectEnabled = false,
                    imeAction = ImeAction.Next
                ),
                onSubmit = { focusManager.moveFocus(FocusDirection.Next) },
                modifier = Modifier.fillMaxWidth(),
            )
            SimpleTextField(
                value = port,
                onValueChange = { port = it.takeIfNumber(port, maxDigits = 5) },
                labelText = stringResource(Res.string.settings_network_port_label),
                keyboardOptions = KeyboardOptions(
                    keyboardType = KeyboardType.Number,
                    imeAction = ImeAction.Done,
                ),
                onSubmit = { focusManager.clearFocus() },
                modifier = Modifier.fillMaxWidth(),
            )
        }
    }
}