package nand.apps.chat.ui.settings.embed

import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material.*
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Add
import androidx.compose.runtime.*
import androidx.compose.ui.Modifier
import nand.apps.chat.ui.provider.LocalSettingsProvider
import nand.apps.chat.ui.settings.*
import nand.apps.chat.ui.settings.row.SettingsListItemRow
import nand.apps.chat.ui.text.SimpleTextFieldButton
import nand.apps.chat.ui.text.rememberTextFieldValue
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.util.getHostName
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun EmbedHostsScreen(
    viewModel: SettingsViewModel,
    modifier: Modifier = Modifier,
) = SettingsColumn(
    title = Res.string.settings_embed_hosts_title,
    modifier = modifier,
    isScrollable = false,
) {
    val state = LocalSettingsProvider.current.embedSettings
    var input by rememberTextFieldValue()
    
    Text(
        text = stringResource(Res.string.settings_embed_hosts_header),
        style = ChatTheme.typography.body1,
        color = ChatTheme.colors.onBackground,
    )
    
    SimpleTextFieldButton(
        value = input,
        hintText = stringResource(Res.string.settings_embed_hosts_hint),
        buttonIcon = Icons.Filled.Add,
        buttonDescription = Res.string.settings_embed_hosts_add_host,
        onValueChange = { input = it },
        onSubmit = {
            val host = it.text.getHostName() ?: it.text.substringBefore('/').trim()
            if (host.isNotBlank() && host !in state.enabledHosts) {
                viewModel.update(state.copy(enabledHosts = state.enabledHosts + host))
            }
        },
        modifier = Modifier
            .fillMaxWidth()
            .padding(vertical = ChatTheme.dimens.spacingMedium),
    )

    val hosts = remember(state.enabledHosts) {
        state.enabledHosts.toList()
    }
    LazyColumn(
        verticalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingMedium),
        modifier = Modifier.fillMaxSize(),
    ) {
        items(hosts) { host ->
            SettingsListItemRow(
                icon = null,
                text = host,
                onDelete = {
                    viewModel.update(state.copy(enabledHosts = state.enabledHosts - host))
                },
                deleteContentDescription = stringResource(Res.string.settings_embed_hosts_remove_host),
            )
        }
    }
}