package nand.apps.chat.ui.settings.filetransfer

import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material.*
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Add
import androidx.compose.runtime.*
import androidx.compose.ui.Modifier
import nand.apps.chat.ui.icons.fileExtensionIcon
import nand.apps.chat.ui.provider.LocalSettingsProvider
import nand.apps.chat.ui.settings.*
import nand.apps.chat.ui.settings.row.SettingsListItemRow
import nand.apps.chat.ui.text.SimpleTextFieldButton
import nand.apps.chat.ui.text.rememberTextFieldValue
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun FileTransferAutoAcceptScreen(
    viewModel: SettingsViewModel,
    modifier: Modifier = Modifier,
) = SettingsColumn(
    title = Res.string.settings_file_transfer_auto_accept_title,
    modifier = modifier,
    isScrollable = false,
) {
    val state = LocalSettingsProvider.current.fileSettings
    var input by rememberTextFieldValue()
    
    Text(
        text = stringResource(Res.string.settings_file_transfer_auto_accept_header),
        style = ChatTheme.typography.body1,
        color = ChatTheme.colors.onBackground,
    )
    
    SimpleTextFieldButton(
        value = input,
        hintText = stringResource(Res.string.settings_file_transfer_auto_accept_hint),
        buttonIcon = Icons.Filled.Add,
        buttonDescription = Res.string.settings_file_transfer_auto_accept_add_rule,
        onValueChange = { input = it },
        onSubmit = {
            val extension = it.text.trimStart('.', '*', ' ').lowercase()
            if (extension.isNotBlank() && extension !in state.autoAcceptFileTypes) {
                viewModel.update(state.copy(autoAcceptFileTypes = state.autoAcceptFileTypes + extension))
            }
        },
        modifier = Modifier
            .fillMaxWidth()
            .padding(vertical = ChatTheme.dimens.spacingMedium),
    )

    val fileTypes = remember(state.autoAcceptFileTypes) {
        state.autoAcceptFileTypes.toList()
    }
    LazyColumn(
        verticalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingMedium),
        modifier = Modifier.fillMaxSize(),
    ) {
        items(fileTypes) { type ->
            SettingsListItemRow(
                icon = fileExtensionIcon(type),
                text = "*.$type",
                onDelete = {
                    viewModel.update(state.copy(autoAcceptFileTypes = state.autoAcceptFileTypes - type))
                },
                deleteContentDescription = stringResource(Res.string.settings_file_transfer_auto_accept_remove_rule),
            )
        }
    }
}