package nand.apps.chat.ui.settings.filetransfer

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.MultipleStop
import androidx.compose.ui.util.fastFilter
import nand.apps.chat.platform.getPlatform
import nand.apps.chat.ui.settings.FileTransferSettingsRoute
import nand.apps.chat.ui.settings.item.SettingsScreenState
import nand.apps.chat.ui.settings.item.SettingsCategory
import nand.apps.chat.ui.settings.item.SettingsSeparatorItem
import seers.composeapp.generated.resources.*

class FileTransferSettingsCategory : SettingsCategory {

    override val title = Res.string.settings_file_transfer_title
    override val description = Res.string.settings_file_transfer_desc
    override val icon = Icons.Outlined.MultipleStop
    override val route = FileTransferSettingsRoute

    private val items = listOf(
        FileTransferDownloadDirectorySettingsItem(),
        FileTransferAvatarsSettingsItem(),
        FileTransferDragSettingsItem(),
        FileTransferPasteSettingsItem(),
        FileTransferGroupUploadSettingsItem(),
        FileTransferAutoAcceptSettingsItem(),
        SettingsSeparatorItem(Res.string.settings_advanced),
        FileTransferUploadEndpointSettingsItem(),
        FileTransferAvatarExpirationSettingsItem(),
        FileTransferConfirmationEnabledSettingsItem(),
    )

    override fun getItems(state: SettingsScreenState) = items.fastFilter { item ->
        when (item) {
            is FileTransferDragSettingsItem,
            is FileTransferPasteSettingsItem,
            is FileTransferGroupUploadSettingsItem,
            is FileTransferDownloadDirectorySettingsItem -> getPlatform().isDesktop
            else -> true
        }
    }
}