package nand.apps.chat.ui.settings.general

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.*
import androidx.compose.runtime.Composable
import nand.apps.chat.locale.setAppLocale
import nand.apps.chat.ui.provider.LocalNavController
import nand.apps.chat.ui.provider.LocalSettingsProvider
import nand.apps.chat.ui.settings.SettingsViewModel
import nand.apps.chat.ui.settings.dialog.LocaleSettingDialog
import nand.apps.chat.ui.settings.item.SettingsItem
import nand.apps.chat.ui.settings.row.DialogRow
import seers.composeapp.generated.resources.*

class AppLocaleSettingsItem : SettingsItem {

    override val title = Res.string.settings_general_locale_title
    override val description = Res.string.settings_general_locale_desc
    override val icon = Icons.Outlined.Language

    @Composable
    override fun GetComposable(viewModel: SettingsViewModel) {
        val state = LocalSettingsProvider.current.generalSettings
        val navController = LocalNavController.current
        DialogRow { onConfirm ->
            LocaleSettingDialog(
                onConfirm = { locale ->
                    onConfirm()
                    setAppLocale(locale)
                    viewModel.update(state.copy(locale = locale))
                    if (navController.canNavigateBack) {
                        // Workaround to force complete re-composition so the text updates take effect
                        navController.navigateBack()
                    }
                },
                onCancel = onConfirm,
            )
        }
    }
}