package nand.apps.chat.ui.settings.row

import androidx.compose.material.Checkbox
import androidx.compose.material.CheckboxDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.NonRestartableComposable
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.vector.ImageVector
import nand.apps.chat.ui.settings.item.SettingsItem
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.StringResource

@Composable
fun SettingsCheckboxRow(
    title: StringResource,
    description: StringResource?,
    icon: ImageVector,
    isChecked: Boolean,
    onCheckedChange: (Boolean) -> Unit,
    modifier: Modifier = Modifier,
    isEnabled: Boolean = true,
    endContent: @Composable () -> Unit = {},
) = SettingsRow(
    title = title,
    description = description,
    icon = icon,
    onClick = { onCheckedChange(!isChecked) },
    isEnabled = isEnabled,
    modifier = modifier,
    endContent = {
        endContent()
        Checkbox(
            checked = isChecked,
            enabled = isEnabled,
            onCheckedChange = onCheckedChange,
            colors = CheckboxDefaults.colors(
                checkmarkColor = ChatTheme.colors.onSecondary,
                checkedColor = ChatTheme.colors.surface3,
            ),
        )
    },
)

@Composable
@NonRestartableComposable
fun SettingsItem.CheckboxRow(
    isChecked: Boolean,
    onCheckedChange: (Boolean) -> Unit,
    modifier: Modifier = Modifier,
    isEnabled: Boolean = true,
    endContent: @Composable () -> Unit = {},
) = SettingsCheckboxRow(
    title = title,
    description = description,
    icon = icon,
    isChecked = isChecked,
    onCheckedChange = onCheckedChange,
    modifier = modifier,
    isEnabled = isEnabled,
    endContent = endContent,
)