package nand.apps.chat.ui.settings.row

import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.vector.ImageVector
import nand.apps.chat.ui.settings.item.SettingsItem
import org.jetbrains.compose.resources.StringResource

@Composable
fun SettingsDialogRow(
    title: StringResource,
    description: StringResource?,
    icon: ImageVector,
    modifier: Modifier = Modifier,
    isEnabled: Boolean = true,
    isLoginRequired: Boolean = false,
    content: @Composable (onConfirm: () -> Unit) -> Unit,
) {
    var isDialogVisible by rememberSaveable { mutableStateOf(false) }
    SettingsRow(
        title = title,
        description = description,
        icon = icon,
        isEnabled = isEnabled,
        isLoginRequired = isLoginRequired,
        onClick = { isDialogVisible = true },
        modifier = modifier,
        endContent = {
            if (isDialogVisible) {
                content { isDialogVisible = false }
            }
        },
    )
}

@Composable
@NonRestartableComposable
fun SettingsItem.DialogRow(
    modifier: Modifier = Modifier,
    isEnabled: Boolean = true,
    isLoginRequired: Boolean = false,
    content: @Composable (onConfirm: () -> Unit) -> Unit,
) = SettingsDialogRow(
    title = title,
    description = description,
    icon = icon,
    modifier = modifier,
    isEnabled = isEnabled,
    isLoginRequired = isLoginRequired,
    content = content,
)