package nand.apps.chat.ui.settings.row

import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.vector.ImageVector
import nand.apps.chat.ui.settings.dialog.SettingsInputDialog
import nand.apps.chat.ui.settings.item.SettingsItem
import org.jetbrains.compose.resources.StringResource

@Composable
fun SettingsInputDialogRow(
    title: StringResource,
    description: StringResource?,
    label: StringResource,
    icon: ImageVector,
    value: String,
    onConfirm: (String) -> Unit,
    modifier: Modifier = Modifier,
    regex: Regex? = null,
    isEnabled: Boolean = true,
) {
    var isDialogVisible by rememberSaveable { mutableStateOf(false) }
    SettingsRow(
        title = title,
        description = description,
        icon = icon,
        onClick = { isDialogVisible = true },
        isEnabled = isEnabled,
        modifier = modifier,
        endContent = {
            if (isDialogVisible) {
                SettingsInputDialog(
                    title = title,
                    label = label,
                    initialValue = value,
                    regex = regex,
                    onConfirm = { value ->
                        isDialogVisible = false
                        onConfirm(value)
                    },
                    onCancel = { isDialogVisible = false },
                )
            }
        },
    )
}

@Composable
@NonRestartableComposable
fun SettingsItem.InputDialogRow(
    label: StringResource,
    value: String,
    onConfirm: (String) -> Unit,
    modifier: Modifier = Modifier,
    regex: Regex? = null,
    isEnabled: Boolean = true,
) = SettingsInputDialogRow(
    title = title,
    description = description,
    icon = icon,
    label = label,
    value = value,
    onConfirm = onConfirm,
    modifier = modifier,
    regex = regex,
    isEnabled = isEnabled,
)