package nand.apps.chat.ui.settings.row

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.LazyItemScope
import androidx.compose.material.Icon
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.Delete
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.vector.ImageVector
import nand.apps.chat.ui.button.IconButton
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme

@Composable
fun LazyItemScope.SettingsListItemRow(
    icon: ImageVector?,
    text: String,
    onDelete: (() -> Unit)? = null,
    deleteContentDescription: String? = null,
) = Row(
    verticalAlignment = Alignment.CenterVertically,
    modifier = Modifier
        .fillMaxWidth()
        .clip(ChatTheme.shapes.medium)
        .background(ChatTheme.colors.surface)
        .padding(start = ChatTheme.dimens.spacingMedium)
        .animateItem()
) {
    icon?.let {
        Icon(
            imageVector = icon,
            contentDescription = text,
            tint = ChatTheme.colors.onSurface,
            modifier = Modifier
                .padding(end = ChatTheme.dimens.spacingMedium)
                .size(ChatTheme.dimens.iconMedium),
        )
    }
    TextLine(
        text = text,
        style = ChatTheme.typography.body1,
        modifier = Modifier.weight(1f),
    )
    onDelete?.let {
        IconButton(
            icon = Icons.Outlined.Delete,
            contentDescription = deleteContentDescription,
            onClick = it,
        )
    }
}