package nand.apps.chat.ui.settings.update

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.*
import androidx.compose.runtime.Composable
import nand.apps.chat.ui.provider.LocalSettingsProvider
import nand.apps.chat.time.HOUR_MILLIS
import nand.apps.chat.ui.settings.SettingsViewModel
import nand.apps.chat.ui.settings.item.*
import nand.apps.chat.ui.settings.row.InputDialogRow
import nand.apps.chat.util.REGEX_NON_DIGITS
import nand.apps.chat.util.toIntOrZero
import seers.composeapp.generated.resources.*

class AppUpdateCheckIntervalSettingsItem : SettingsItem {

    override val title = Res.string.settings_app_updates_check_interval_title
    override val description = Res.string.settings_app_updates_check_interval_desc
    override val icon = Icons.Outlined.Schedule

    @Composable
    override fun GetComposable(viewModel: SettingsViewModel) {
        val state = LocalSettingsProvider.current.appUpdateSettings
        InputDialogRow(
            label = Res.string.settings_file_transfer_avatar_expiration_label,
            value = (state.updateCheckInterval / HOUR_MILLIS).toString(),
            regex = REGEX_NON_DIGITS,
            isEnabled = state.isAutoUpdateEnabled,
            onConfirm = { hours ->
                viewModel.update(state.copy(updateCheckInterval = hours.toIntOrZero() * HOUR_MILLIS))
            }
        )
    }
}