package nand.apps.chat.ui.settings.update

import androidx.compose.foundation.layout.size
import androidx.compose.material.CircularProgressIndicator
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.*
import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.Modifier
import nand.apps.chat.BuildConfig.APP_NAME
import nand.apps.chat.model.event.ErrorSnackbarEvent
import nand.apps.chat.model.event.SuccessSnackbarEvent
import nand.apps.chat.model.update.AppReleaseInfo
import nand.apps.chat.res.FormattedStringResource
import nand.apps.chat.ui.provider.LocalEventHandler
import nand.apps.chat.ui.settings.SettingsViewModel
import nand.apps.chat.ui.settings.item.*
import nand.apps.chat.ui.settings.row.Row
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.ui.update.AppUpdateDialog
import nand.apps.chat.update.AppUpdater
import nand.apps.chat.util.LOCAL_CLIENT_VERSION
import org.koin.compose.getKoin
import seers.composeapp.generated.resources.*

class AppUpdateCheckSettingsItem : SettingsItem {

    override val title = Res.string.settings_app_updates_check_title
    override val description = Res.string.settings_app_updates_check_desc
    override val icon = Icons.Outlined.SystemUpdateAlt

    @Composable
    override fun GetComposable(viewModel: SettingsViewModel) {
        val eventHandler = LocalEventHandler.current
        val appUpdater: AppUpdater by getKoin().inject()
        var isCheckingForUpdate by rememberSaveable { mutableStateOf(false) }
        var release by remember { mutableStateOf<AppReleaseInfo?>(null) }
        LaunchedEffect(isCheckingForUpdate) {
            if (isCheckingForUpdate) {
                val latest = appUpdater.getLatestReleaseInfo()
                release = when {
                    latest == null -> {
                        eventHandler += ErrorSnackbarEvent(FormattedStringResource(Res.string.app_updater_check_failed))
                        null
                    }
                    latest.version > LOCAL_CLIENT_VERSION -> latest
                    else -> {
                        eventHandler += SuccessSnackbarEvent(FormattedStringResource(Res.string.app_updater_up_to_date, APP_NAME))
                        null
                    }
                }
                isCheckingForUpdate = false
            }
        }
        Row(
            isEnabled = !isCheckingForUpdate,
            endContent = {
                if (isCheckingForUpdate) {
                    CircularProgressIndicator(
                        modifier = Modifier.size(ChatTheme.dimens.iconSmall),
                    )
                }
            },
            onClick = { isCheckingForUpdate = true },
        )
        release?.let {
            AppUpdateDialog(
                release = it,
                autoDownload = false,
                onCancel = { release = null },
            )
        }
    }
}