package nand.apps.chat.ui.snackbar

import androidx.compose.material.SnackbarHostState
import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.Modifier
import nand.apps.chat.model.event.*
import nand.apps.chat.res.StringResourceProvider
import nand.apps.chat.ui.provider.LocalEventHandler
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.ui.util.LaunchedCollect
import org.koin.compose.getKoin

@Composable
fun GlobalSnackbarHost(
    modifier: Modifier = Modifier,
) {
    val eventHandler = LocalEventHandler.current
    val stringProvider: StringResourceProvider by getKoin().inject()
    var isError by rememberSaveable { mutableStateOf(false) }
    val errorSnackbarState = remember { SnackbarHostState() }
    val successSnackbarState = remember { SnackbarHostState() }
    LaunchedCollect(eventHandler.onEvent) { event ->
        when (event) {
            is ErrorSnackbarEvent -> {
                isError = true
                errorSnackbarState.showSnackbar(stringProvider.getString(event.message))
            }
            is SuccessSnackbarEvent -> {
                isError = false
                successSnackbarState.showSnackbar(stringProvider.getString(event.message))
            }
            else -> Unit
        }
    }
    if (isError) {
        SimpleSnackbarHost(
            state = errorSnackbarState,
            backgroundColor = ChatTheme.colors.error,
            modifier = modifier,
        )
    } else {
        SimpleSnackbarHost(
            state = successSnackbarState,
            backgroundColor = ChatTheme.colors.secondary,
            modifier = modifier,
        )
    }
}