package nand.apps.chat.ui.text

import androidx.compose.foundation.interaction.MutableInteractionSource
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.material.ExperimentalMaterialApi
import androidx.compose.material.TextFieldDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.remember
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.input.VisualTransformation
import nand.apps.chat.ui.modifier.onClick
import nand.apps.chat.ui.theme.ChatTheme

@OptIn(ExperimentalMaterialApi::class)
@Composable
fun ClickableTextField(
    text: String,
    onClick: () -> Unit,
    modifier: Modifier = Modifier,
    label: String? = null,
    isEnabled: Boolean = true,
) {
    val interactionSource = remember { MutableInteractionSource() }
    val colors = TextFieldDefaults.outlinedTextFieldColors()
    Box(
        modifier = modifier
            .padding(top = ChatTheme.dimens.spacingMedium)
            .onClick(onClick = onClick, isEnabled = isEnabled),
    ) {
        TextFieldDefaults.OutlinedTextFieldDecorationBox(
            value = text,
            visualTransformation = VisualTransformation.None,
            innerTextField = {
                TextLine(
                    text = text,
                    style = ChatTheme.typography.body2,
                    color = ChatTheme.colors.onSurface,
                    modifier = Modifier
                        .fillMaxWidth()
                        .padding(vertical = ChatTheme.dimens.spacingXSmall),
                )
            },
            label = label?.let {
                {
                    TextLine(
                        text = it,
                        style = ChatTheme.typography.body2,
                        color = ChatTheme.colors.onBackground2,
                    )
                }
            },
            singleLine = true,
            enabled = false,
            isError = false,
            interactionSource = interactionSource,
            border = {
                TextFieldDefaults.BorderBox(
                    enabled = false,
                    isError = false,
                    interactionSource = interactionSource,
                    colors = colors,
                )
            },
        )
    }
}