package nand.apps.chat.ui.text

import androidx.compose.ui.text.input.TransformedText
import androidx.compose.ui.util.fastAll
import nand.apps.chat.text.UnicodeFont
import nand.apps.chat.util.fastForEach

const val UNICODE_FONT_TAG = "ufont"

fun TransformedText.applyFontAnnotations(): TransformedText {
    val src = text
    val fontAnnotations = src.getStringAnnotations(UNICODE_FONT_TAG, 0, src.length)
    if (fontAnnotations.isEmpty() || fontAnnotations.fastAll { it.item == UnicodeFont.DEFAULT.name }) {
        return this
    }
    return buildTransformedText(this) {
        val fontsPerChar = arrayOfNulls<UnicodeFont>(src.length)
        fontAnnotations.fastForEach { ann ->
            val font = UnicodeFont.valueOf(ann.item)
            fontsPerChar.fill(font, ann.start, ann.end)
        }
        src.text.forEachIndexed { index, char ->
            val font = fontsPerChar[index]
            val newChar = font?.charMap?.get(char) ?: char.toString()
            appendReplacement(newChar, index, index + 1)
        }
    }
}