package nand.apps.chat.ui.text

import androidx.compose.ui.text.input.OffsetMapping
import nand.apps.chat.util.fastForEach

class IndexOffsetMapping(
    indices: List<Pair<Int, Int>>,
    indexMap: Map<Int, Int> = indices.toMap(),
    previousMapping: OffsetMapping = OffsetMapping.Identity,
) : OffsetMapping {

    private val srcToDst: Map<Int, Int>
    private val dstToSrc: Map<Int, Int>

    init {
        when {
            previousMapping is IndexOffsetMapping -> {
                srcToDst = LinkedHashMap(previousMapping.srcToDst)
                previousMapping.srcToDst.forEach { (src, dst) ->
                    srcToDst[src] = indexMap[dst] ?: missingMapping(dst)
                }
                dstToSrc = mutableMapOf()
                var lastKey = -1
                indices.forEach{ (src, dst) ->
                    for (i in lastKey + 1..<dst) {
                        dstToSrc[i] = dstToSrc[lastKey] ?: 0
                    }
                    dstToSrc[dst] = (previousMapping.dstToSrc[src] ?: missingMapping(src))
                    lastKey = dst
                }
            }
            previousMapping === OffsetMapping.Identity -> {
                srcToDst = indexMap
                dstToSrc = indices.associate { it.second to it.first }
            }
            else -> {
                srcToDst = mutableMapOf()
                dstToSrc = mutableMapOf()
                indices.fastForEach { (_, dst) ->
                    val prevSrc = previousMapping.transformedToOriginal(dst)
                    srcToDst[prevSrc] = dst
                    dstToSrc[dst] = prevSrc
                }
            }
        }
        srcToDst.checkNoGaps()
        dstToSrc.checkNoGaps()
    }

    override fun originalToTransformed(offset: Int) =
        srcToDst[offset] ?: missingOffset(offset)

    override fun transformedToOriginal(offset: Int) =
        dstToSrc[offset] ?: missingOffset(offset)

    private fun missingMapping(index: Int): Nothing = error("Missing mapping $index")

    private fun missingOffset(offset: Int): Nothing = error("Missing offset $offset")

    private fun Map<Int, Int>.checkNoGaps() {
        check(keys.last() == size - 1) { "Map contains gaps: ${keys.joinToString()}" }
    }
}