package nand.apps.chat.ui.text

import androidx.compose.foundation.layout.*
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.vector.ImageVector
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.input.TextFieldValue
import androidx.compose.ui.unit.dp
import nand.apps.chat.ui.button.StyledIconButton
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.StringResource

@Composable
fun SimpleTextFieldButton(
    value: TextFieldValue,
    buttonIcon: ImageVector,
    modifier: Modifier = Modifier,
    buttonDescription: StringResource,
    textStyle: TextStyle = ChatTheme.typography.body2,
    hintText: String? = null,
    labelText: String? = null,
    onValueChange: (TextFieldValue) -> Unit = {},
    onSubmit: (TextFieldValue) -> Unit = {},
) {
    Row(
        verticalAlignment = Alignment.CenterVertically,
        modifier = modifier,
    ) {
        SimpleTextField(
            value = value,
            hintText = hintText,
            labelText = labelText,
            textStyle = textStyle,
            onSubmit = onSubmit,
            onValueChange = onValueChange,
            modifier = Modifier
                .weight(1f)
                .padding(end = ChatTheme.dimens.spacingSmall)
        )
        SimpleTextFieldButton(
            buttonIcon = buttonIcon,
            buttonDescription = buttonDescription,
            onClick = { onSubmit(value) },
            modifier = Modifier
                .padding(top = labelText?.let { ChatTheme.dimens.spacingMedium } ?: 0.dp)
        )
    }
}

@Composable
fun SimpleTextFieldButton(
    buttonIcon: ImageVector,
    onClick: () -> Unit,
    modifier: Modifier = Modifier,
    buttonDescription: StringResource,
) = StyledIconButton(
    icon = buttonIcon,
    contentDescription = buttonDescription,
    onClick = onClick,
    modifier = modifier.size(ChatTheme.dimens.buttonXLarge),
)