package nand.apps.chat.ui.text

import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.NonRestartableComposable
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.AnnotatedString
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.text.style.TextOverflow
import nand.apps.chat.ui.modifier.shadow
import nand.apps.chat.ui.theme.ChatTheme

@Composable
@NonRestartableComposable
fun TextLine(
    text: String,
    style: TextStyle,
    modifier: Modifier = Modifier,
    color: Color = ChatTheme.colors.onSurface,
    textAlign: TextAlign? = null,
    overflow: TextOverflow = TextOverflow.Ellipsis,
    hasShadow: Boolean = false,
    shadowColor: Color = Color.Black,
) = TextLine(
    text = AnnotatedString(text),
    style = style,
    modifier = modifier,
    color = color,
    textAlign = textAlign,
    overflow = overflow,
    hasShadow = hasShadow,
    shadowColor = shadowColor,
)

@Composable
@NonRestartableComposable
fun TextLine(
    text: AnnotatedString,
    style: TextStyle,
    modifier: Modifier = Modifier,
    color: Color = ChatTheme.colors.onSurface,
    textAlign: TextAlign? = null,
    overflow: TextOverflow = TextOverflow.Ellipsis,
    hasShadow: Boolean = false,
    shadowColor: Color = Color.Black,
) {
    if (hasShadow) {
        Text(
            text = text,
            color = shadowColor,
            style = style,
            maxLines = 1,
            overflow = overflow,
            textAlign = textAlign,
            modifier = modifier.shadow(),
        )
    }
    Text(
        text = text,
        color = color,
        style = style,
        maxLines = 1,
        overflow = overflow,
        textAlign = textAlign,
        modifier = modifier,
    )
}