package nand.apps.chat.ui.theme

import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.MaterialTheme
import androidx.compose.material.Shapes
import androidx.compose.material.Typography
import androidx.compose.runtime.*
import androidx.compose.ui.unit.dp

private val LocalColors = staticCompositionLocalOf { DarkChatColors }
private val LocalDimensions = staticCompositionLocalOf { ChatThemeDimensions() }

object ChatTheme {

    val colors: ChatThemeColors
        @Composable
        @ReadOnlyComposable
        get() = LocalColors.current

    val typography: Typography
        @Composable
        @ReadOnlyComposable
        get() = MaterialTheme.typography
    
    val shapes: Shapes
        @Composable
        @ReadOnlyComposable
        get() = MaterialTheme.shapes
    
    val dimens: ChatThemeDimensions
        @Composable
        @ReadOnlyComposable
        get() = LocalDimensions.current
}

@Composable
fun ChatTheme(
    colors: ChatThemeColors = DarkChatColors,
    content: @Composable () -> Unit,
) {
    val providers = getChatThemeProviders(colors) + (LocalColors provides colors)
    CompositionLocalProvider(*providers.toTypedArray()) {
        MaterialTheme(
            colors = colors.material,
            shapes = Shapes(
                small = RoundedCornerShape(6.dp),
            ),
            content = content,
        )
    }
}

expect fun getChatThemeProviders(colors: ChatThemeColors): List<ProvidedValue<*>>