package nand.apps.chat.ui.update

import com.hoc081098.kmp.viewmodel.ViewModel
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.launch
import nand.apps.chat.update.AppInstaller
import nand.apps.chat.update.AppUpdater
import okio.Path.Companion.toPath

class AppUpdateDialogViewModel(
    private val appUpdater: AppUpdater,
    private val appInstaller: AppInstaller,
    initialState: AppUpdateDialogState,
) : ViewModel() {

    val stateData = MutableStateFlow(initialState)
    private val state get() = stateData.value

    fun onConfirm() {
        if (state.isDownloading) return
        state.packagePath?.let { install() } ?: viewModelScope.launch { download() }
    }

    suspend fun download() {
        updateState {
            it.copy(
                isDownloading = true,
                isDownloadFailure = false,
                isInstallFailure = false,
            )
        }
        var isDownloadFailure = false
        val packagePath = appUpdater.download(
            request = state.release.download,
            onProgress = { progress ->
                updateState { it.copy(downloadProgress = progress) }
            },
        )?.toString() ?: run {
            isDownloadFailure = true
            null
        }
        updateState {
            it.copy(
                packagePath = packagePath,
                isDownloading = false,
                isDownloadFailure = isDownloadFailure
            )
        }
    }

    private fun install() {
        val path = state.packagePath?.toPath() ?: return
        viewModelScope.launch {
            appInstaller.install(
                release = state.release,
                packagePath = path,
            )
            // The install() call above never returns unless something went wrong
            updateState { it.copy(isInstallFailure = true) }
        }
    }

    private fun updateState(block: (AppUpdateDialogState) -> AppUpdateDialogState) {
        stateData.tryEmit(block(state))
    }
}