package nand.apps.chat.audio

import nand.apps.audio.AudioFileWriter
import nand.apps.chat.model.audio.AudioMetadata
import okio.FileSystem
import okio.Path.Companion.toPath
import okio.buffer
import okio.use

class TestAudioFileWriter(
    val path: String,
    val sampleRate: Int,
    val channels: Int,
    val bitRate: Int,
    val metadata: AudioMetadata? = null,
    private val fileSystem: FileSystem,
) : AudioFileWriter {

    init {
        fileSystem.createDirectories(path.toPath().parent!!)
        fileSystem.write(path.toPath()) {
            writeInt(sampleRate)
            writeInt(channels)
            writeInt(bitRate)
            metadata?.run {
                title?.let { write(it.encodeToByteArray()) }
                artist?.let { write(it.encodeToByteArray()) }
            }
        }
    }

    override fun write(buffer: ShortArray, offset: Int, length: Int): Int {
        fileSystem.appendingSink(path.toPath()).buffer().use { stream ->
            for (i in offset..<offset + length) {
                stream.writeShort(buffer[i].toInt())
            }
        }
        return length
    }

    override fun close() {
    }
}