package nand.apps.chat.audio.id3

import nand.apps.chat.di.initKoinTest
import nand.apps.chat.util.*
import okio.FileSystem
import org.koin.core.context.stopKoin
import kotlin.test.AfterTest
import kotlin.test.BeforeTest
import kotlin.test.Test
import kotlin.test.assertEquals

class Id3TagReaderTests {

    private lateinit var tagReader: Id3TagReader
    private lateinit var fileSystem: FileSystem

    @BeforeTest
    fun init() {
        initKoinTest().koin.apply {
            tagReader = get()
            fileSystem = get()
        }
    }

    @AfterTest
    fun teardown() {
        stopKoin()
    }

    @Test
    fun `Read ID3v2 tags`() {
        fileSystem.writeTestResource(MP3_SAMPLE_V2)
        val tags = tagReader.readTags(MP3_SAMPLE_V2)
        assertEquals(
            expected = Id3TagData(
                title = MP3_SAMPLE_V2_METADATA.title,
                artist = MP3_SAMPLE_V2_METADATA.artist,
                version = "ID3v2.4.0",
            ),
            actual = tags,
        )
    }

    @Test
    fun `Read ID3v1 tags`() {
        fileSystem.writeTestResource(MP3_SAMPLE_V1)
        val tags = tagReader.readTags(MP3_SAMPLE_V1)
        assertEquals(
            expected = Id3TagData(
                title = MP3_SAMPLE_V1_METADATA.title,
                artist = MP3_SAMPLE_V1_METADATA.artist,
                version = "ID3v1",
            ),
            actual = tags,
        )
    }
}