package nand.apps.chat.ui.settings.net.tox

import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.ExperimentalCoroutinesApi
import kotlinx.coroutines.test.UnconfinedTestDispatcher
import kotlinx.coroutines.test.resetMain
import kotlinx.coroutines.test.runTest
import kotlinx.coroutines.test.setMain
import nand.apps.chat.di.initKoinTest
import nand.apps.chat.engine.tox.*
import nand.apps.chat.ui.viewmodel.ViewModelTestScope
import kotlin.test.*

private typealias ToxNodeSettingsViewModelTestScope = ViewModelTestScope<ToxNodeSettingsViewModel, ToxNodeSettingsState, Unit>

@OptIn(ExperimentalCoroutinesApi::class)
class ToxNodesSettingsViewModelTests {

    private lateinit var scope: ToxNodeSettingsViewModelTestScope

    @BeforeTest
    fun setup() {
        Dispatchers.setMain(UnconfinedTestDispatcher())
    }

    @AfterTest
    fun teardown() {
        if (this::scope.isInitialized) scope.destroy()
        Dispatchers.resetMain()
    }

    @Test
    fun `Refresh Tox nodes screen and view details`() = viewModelTest {
        // Refresh nodes
        runTest { viewModel.refresh() }
        val state = assertState(
            ToxNodeSettingsState(
                isLoading = false,
                bootstrapData = TOX_BOOTSTRAP_DATA.copy(
                    nodes = listOf(
                        TOX_NODE_1,
                        TOX_NODE_3,
                        TOX_NODE_2,
                    )
                ),
            )
        )

        // View details
        viewModel.showDetailsDialog(TOX_NODE_2)
        assertState(state.copy(detailsNode = TOX_NODE_2))
        viewModel.dismissDetailsDialog()
        assertState(state)

        // Show refresh dialog
        viewModel.showRefreshDialog()
        assertState(state.copy(isRefreshDialogVisible = true))
        viewModel.updateNodes("seers.chat")
        assertEquals(
            expected = "seers.chat",
            actual = settingsRepo.settings.networkSettings.toxNodesEndpointUrl,
        )
        assertState(
            state.copy(
                isRefreshDialogVisible = false,
                isLoading = true,
            )
        )
        assertState(state)
    }

    private fun viewModelTest(block: ToxNodeSettingsViewModelTestScope.() -> Unit) = runTest {
        val koin = initKoinTest().koin
        scope = ToxNodeSettingsViewModelTestScope(koin)
        val viewModel: ToxNodeSettingsViewModel = koin.get()
        scope.initTestScope(
            scope = this,
            viewModel = viewModel,
            stateData = viewModel.stateData,
        )
        with(scope) {
            assertState(ToxNodeSettingsState())
            block()
        }
    }
}