package nand.apps.chat.ui.update

import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.ExperimentalCoroutinesApi
import kotlinx.coroutines.test.*
import nand.apps.chat.di.initKoinTest
import nand.apps.chat.ui.viewmodel.ViewModelTestScope
import nand.apps.chat.util.APP_RELEASE_INFO
import org.koin.core.parameter.parametersOf
import kotlin.test.AfterTest
import kotlin.test.BeforeTest
import kotlin.test.Test

private typealias AppUpdateDialogViewModelTestScope = ViewModelTestScope<AppUpdateDialogViewModel, AppUpdateDialogState, Unit>

@OptIn(ExperimentalCoroutinesApi::class)
class AppUpdateDialogViewModelTests {

    private lateinit var scope: AppUpdateDialogViewModelTestScope

    @BeforeTest
    fun setup() {
        Dispatchers.setMain(UnconfinedTestDispatcher())
    }

    @AfterTest
    fun teardown() {
        if (this::scope.isInitialized) scope.destroy()
        Dispatchers.resetMain()
    }

    @Test
    fun `App download and install successful`() = viewModelTest { initialState ->
        // Start download
        onDelay {
            runTest { /* Force state collection */ }
        }
        viewModel.onConfirm()
        var state = assertState(
            initialState.copy(
                isDownloading = true,
                isDownloadFailure = false,
                isInstallFailure = false,
            )
        )
        for (progress in 25..100 step 25) {
            state = assertState(state.copy(downloadProgress = progress))
        }
        state = assertState(
            state.copy(
                packagePath = "downloads/installer.deb",
                isDownloading = false,
                isDownloadFailure = false,
            )
        )

        // Install package (and fail because the app doesn't quit)
        viewModel.onConfirm()
        assertState(state.copy(isInstallFailure = true))
    }

    private fun viewModelTest(
        block: AppUpdateDialogViewModelTestScope.(AppUpdateDialogState) -> Unit,
    ) = runTest {
        val koin = initKoinTest().koin
        val initialState = AppUpdateDialogState(release = APP_RELEASE_INFO)
        scope = AppUpdateDialogViewModelTestScope(koin)
        val viewModel: AppUpdateDialogViewModel = koin.get(parameters = { parametersOf(initialState, scope.delayScope) })
        scope.initTestScope(
            scope = this,
            viewModel = viewModel,
            stateData = viewModel.stateData,
        )
        with(scope) {
            assertState(initialState)
            block(initialState)
        }
    }
}