package nand.apps.chat.util

import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.launch
import kotlinx.coroutines.test.TestScope
import kotlin.test.assertEquals

fun <T> TestScope.collect(flow: Flow<T>): Channel<T> {
    val channel = Channel<T>(Channel.UNLIMITED)
    backgroundScope.launch(Dispatchers.Unconfined) {
        flow.collect { channel.send(it) }
    }
    return channel
}

fun <T> Channel<T>.get() = tryReceive().getOrNull()

fun <T> Channel<T>.assertValue(expected: T?) = assertEquals(
    expected = expected,
    actual = get(),
)