package nand.apps.chat.res

import org.jetbrains.compose.resources.StringArrayResource
import org.jetbrains.compose.resources.StringResource
import java.io.File

private val REGEX_STRING_ELEMENT = Regex("^<string name=\"([\\w_]+)\".*>(.*)</string>$")
private val REGEX_STRING_ARRAY_ELEMENT = Regex("^<string-array name=\"([\\w_]+)\".*>$")
private val REGEX_ITEM_ELEMENT = Regex("^<item>(.*)</item>$")

class HeadlessStringResourceProvider : StringResourceProvider {

    private var resourcesLoaded = false
    private val stringMap = mutableMapOf<String, String>()
    private val stringArrayMap = mutableMapOf<String, List<String>>()

    override fun getString(resource: StringResource, vararg args: Any): String {
        if (!resourcesLoaded) loadStrings()
        return stringMap[resource.key]?.format(args) ?: error("Failed to find string ${resource.key}")
    }

    override fun getStringArray(resource: StringArrayResource): List<String> {
        if (!resourcesLoaded) loadStrings()
        return stringArrayMap[resource.key] ?: error("Failed to find string array resource ${resource.key}")
    }

    private fun loadStrings() {
        // TODO - Support localization
        val stringsXml = File(requireDesktopResourceDir(), "headlessResources/values/strings.xml")
        var stringArrayKey: String? = null
        var stringArray = mutableListOf<String>()
        stringsXml.forEachLine {
            val line = it.trim()
            when {
                line.startsWith("<string ") -> REGEX_STRING_ELEMENT.find(line)?.let { match ->
                    val (_, key, value) = match.groupValues
                    stringMap[key] = value
                }
                line.startsWith("<string-array ") -> REGEX_STRING_ARRAY_ELEMENT.find(line)?.let { match ->
                    stringArrayKey = match.groupValues.last()
                    stringArray = mutableListOf()
                }
                line == "</string-array>" -> {
                    stringArrayKey?.let { key ->
                        stringArrayMap[key] = stringArray
                    }
                    stringArrayKey = null
                }
                line.startsWith("<item>") -> REGEX_ITEM_ELEMENT.find(line)?.let { match ->
                    stringArray += match.groupValues.last()
                }
            }
        }
        resourcesLoaded = true
    }
}