package nand.apps.chat.ui.image

import androidx.compose.foundation.Image
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.*
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.layout.onSizeChanged
import androidx.compose.ui.unit.IntSize
import coil3.decode.DecodeUtils
import coil3.size.Scale
import com.sksamuel.scrimage.ImmutableImage
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import org.jetbrains.compose.resources.DrawableResource
import org.jetbrains.compose.resources.imageResource
import kotlin.math.roundToInt

@Composable
actual fun PlatformImage(
    resource: DrawableResource,
    contentDescription: String?,
    modifier: Modifier,
    alignment: Alignment,
    contentScale: ContentScale,
    alpha: Float,
    colorFilter: ColorFilter?,
    filterQuality: FilterQuality,
) {
    val baseBitmap = imageResource(resource)
    var scaledBitmap by remember(baseBitmap) { mutableStateOf<ImageBitmap?>(null) }
    var size by remember { mutableStateOf(IntSize(0, 0)) }
    LaunchedEffect(baseBitmap, size, contentScale) {
        val (width, height) = when (contentScale) {
            ContentScale.Fit,
            ContentScale.Crop -> {
                val scale = DecodeUtils.computeSizeMultiplier(
                    srcWidth = baseBitmap.width,
                    srcHeight = baseBitmap.height,
                    dstWidth = size.width,
                    dstHeight = size.height,
                    scale = if (contentScale == ContentScale.Crop) Scale.FILL else Scale.FIT,
                )
                (baseBitmap.width * scale).roundToInt() to (baseBitmap.height * scale).roundToInt()
            }
            else -> error("Unsupported")
        }
        if (width > 0 && height > 0) {
            scaledBitmap = withContext(Dispatchers.IO) {
                ImmutableImage.fromAwt(baseBitmap.toAwtImage())
                    .scaleTo(width, height)
                    .awt()
                    .toComposeImageBitmap()
            }
        }
    }
    Image(
        bitmap = scaledBitmap ?: baseBitmap,
        contentDescription = contentDescription,
        alignment = alignment,
        contentScale = contentScale,
        alpha = alpha,
        colorFilter = colorFilter,
        filterQuality = filterQuality,
        modifier = modifier.onSizeChanged { size = it },
    )
}