package nand.apps.chat.ui.notification

import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.padding
import androidx.compose.runtime.Composable
import androidx.compose.runtime.CompositionLocalProvider
import androidx.compose.ui.Modifier
import nand.apps.chat.model.notification.MessageNotification
import nand.apps.chat.ui.chat.message.CompactMessageBlock
import nand.apps.chat.ui.contacts.ContactsState
import nand.apps.chat.ui.contacts.LocalContactsState
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme

@Composable
fun MessageNotificationRenderer(
    notification: MessageNotification,
    state: DesktopNotificationState,
) = with(notification) {
    CompositionLocalProvider(
        LocalContactsState provides ContactsState(
            conversationUid = message.conversationUid,
            friends = state.friends,
            groups = state.groups,
        ),
    ) {
        Column(
            verticalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingSmall),
        ) {
            TextLine(
                text = title,
                style = ChatTheme.typography.body1,
                modifier = Modifier.padding(end = ChatTheme.dimens.iconMedium),
            )
            CompactMessageBlock(
                message = message,
                avatar = avatar,
                displayName = sender,
            )
        }
    }
}