package nand.apps.chat.ui.util

import androidx.compose.ui.ExperimentalComposeUiApi
import androidx.compose.ui.draganddrop.DragAndDropEvent
import androidx.compose.ui.draganddrop.DragData
import androidx.compose.ui.draganddrop.awtTransferable
import androidx.compose.ui.draganddrop.dragData
import nand.apps.chat.io.FILE_SCHEME
import nand.apps.chat.log.getLogger
import nand.apps.chat.platform.ChatPlatform
import nand.apps.chat.platform.getPlatform
import net.thauvin.erik.urlencoder.UrlEncoderUtil
import java.awt.datatransfer.DataFlavor

private val logger = getLogger("DragDropUtil")

@OptIn(ExperimentalComposeUiApi::class)
actual fun DragAndDropEvent.getFilePath(): String? {
    val filePath = try {
        when {
            awtTransferable.isDataFlavorSupported(DataFlavor.stringFlavor) -> {
                (awtTransferable.getTransferData(DataFlavor.stringFlavor) as? String)?.run {
                    if (startsWith(FILE_SCHEME)) {
                        substringAfter(FILE_SCHEME).substringBefore('\n')
                    } else {
                        null
                    }
                }
            }
            else -> (dragData() as? DragData.FilesList)?.readFiles()?.firstOrNull()?.run {
                UrlEncoderUtil.decode(substringAfter("file:"))
            }
        }
    } catch (e: Exception) {
        logger.error("Failed to parse drag-and-drop event", e)
        null
    }
    return filePath?.run {
        if (getPlatform() == ChatPlatform.WINDOWS) {
            trimStart('/')
        } else {
            this
        }
    }
}