package nand.apps.chat.ui.widget

import androidx.compose.runtime.Composable
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.ui.graphics.painter.Painter
import androidx.compose.ui.window.ApplicationScope
import androidx.compose.ui.window.MenuScope
import androidx.compose.ui.window.Tray
import androidx.compose.ui.window.rememberTrayState
import com.hoc081098.kmp.viewmodel.koin.compose.koinKmpViewModel
import nand.apps.chat.BuildConfig.APP_NAME
import nand.apps.chat.model.user.UserStatus
import nand.apps.chat.ui.provider.LocalLoginState
import org.jetbrains.compose.resources.StringResource
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun ApplicationScope.ChatSystemTrayIcon(
    windowIcon: Painter,
    onClose: () -> Unit,
    viewModel: ChatTrayIconViewModel = koinKmpViewModel(),
) {
    val state by viewModel.stateData.collectAsState()
    val isLoggedIn = LocalLoginState.current

    if (!state.settings.isEnabled) return

    val trayState = rememberTrayState()
    Tray(
        icon = windowIcon,
        state = trayState,
        tooltip = APP_NAME,
        menu = {
            if (isLoggedIn) {
                StatusMenuItem(
                    status = UserStatus.DEFAULT,
                    label = Res.string.status_online,
                    state = state,
                    viewModel = viewModel,
                )
                StatusMenuItem(
                    status = UserStatus.AWAY,
                    label = Res.string.status_away,
                    state = state,
                    viewModel = viewModel,
                )
                StatusMenuItem(
                    status = UserStatus.BUSY,
                    label = Res.string.status_busy,
                    state = state,
                    viewModel = viewModel,
                )
            }
            Item(
                text = stringResource(Res.string.close),
                onClick = onClose,
            )
        },
        onAction = {
            viewModel.toggleAppWindow()
        },
    )
}

@Composable
private fun MenuScope.StatusMenuItem(
    status: UserStatus,
    label: StringResource,
    state: ChatTrayIconState,
    viewModel: ChatTrayIconViewModel,
) {
    Item(
        text = stringResource(label) + when {
            state.status == status -> " ✓"
            else -> ""
        },
        onClick = { viewModel.setSelfStatus(status) },
    )
}