package nand.apps.chat.ui.widget

import com.hoc081098.kmp.viewmodel.ViewModel
import kotlinx.coroutines.flow.MutableStateFlow
import nand.apps.chat.event.ChatEventHandler
import nand.apps.chat.model.event.RequestAppFocusEvent
import nand.apps.chat.model.event.SetSelfStatusEvent
import nand.apps.chat.model.settings.notifications.ChatTraySettingsData
import nand.apps.chat.model.user.UserStatus
import nand.apps.chat.repo.SelfProfileRepo
import nand.apps.chat.repo.SettingsRepo
import nand.apps.chat.ui.util.collect

class ChatTrayIconViewModel(
    settingsRepo: SettingsRepo,
    selfProfileRepo: SelfProfileRepo,
    private val eventHandler: ChatEventHandler,
) : ViewModel() {

    val stateData = MutableStateFlow(ChatTrayIconState())
    private val state get() = stateData.value

    init {
        collect(settingsRepo.observer) { settings ->
            onSettingsChanged(settings.notificationSettings.traySettings)
        }
        collect(selfProfileRepo.selfObserver) { self ->
            updateState { it.copy(status = self.status) }
        }
    }

    fun toggleAppWindow() {
        eventHandler += RequestAppFocusEvent(
            toggle = true,
            isTrayIconEvent = true,
        )
    }

    fun setSelfStatus(status: UserStatus) {
        eventHandler += SetSelfStatusEvent(status)
    }

    private fun onSettingsChanged(settings: ChatTraySettingsData) {
        updateState { it.copy(settings = settings) }
    }

    private fun updateState(block: (ChatTrayIconState) -> ChatTrayIconState) {
        stateData.tryEmit(block(state))
    }
}