package nand.apps.chat.model.call

import nand.apps.chat.model.event.*
import nand.apps.chat.model.uid.ContactUid

enum class CallAction {
    ANSWER,
    DECLINE,
    HANG_UP,
    PAUSE,
    RESUME,
    ;
    
    fun getEvent(contactUid: ContactUid) = when (this) {
        ANSWER -> AnswerCallEvent(
            contactUid = contactUid,
            isAudioEnabled = true,
            isVideoEnabled = false, // TODO - Support video calling
        )
        DECLINE,
        HANG_UP -> EndCallEvent(contactUid)
        PAUSE -> PauseCallEvent(contactUid)
        RESUME -> ResumeCallEvent(contactUid)
    }
}

fun CallStatus.getActions() = when (this) {
    CallStatus.INCOMING,
    CallStatus.VOICEMAIL -> listOf(CallAction.ANSWER, CallAction.DECLINE)
    CallStatus.OUTGOING -> listOf(CallAction.HANG_UP)
    CallStatus.ACTIVE -> listOf(CallAction.PAUSE, CallAction.HANG_UP)
    CallStatus.PAUSED -> listOf(CallAction.RESUME, CallAction.HANG_UP)
    else -> emptyList()
}