package nand.apps.chat.model.file

import io.ktor.http.*
import nand.apps.chat.platform.ChatPlatform
import nand.apps.chat.platform.getPlatform

val ContentType.Audio.WAV get() = ContentType("audio", "wav")
val ContentType.Audio.M4A get() = ContentType("audio", "x-m4a")
val ContentType.Audio.FLAC get() = ContentType("audio", "flac")
val ContentType.Audio.OPUS get() = ContentType("audio", "opus")
val ContentType.Application.OGG get() = ContentType("application", "ogg")

fun ContentType.isSupportedAudioFormat() = when (this) {
    // All platforms support Opus and MP3
    ContentType.Audio.OPUS,
    ContentType.Audio.MPEG -> true

    // Android supports a bunch of additional formats
    ContentType.Audio.WAV,
    ContentType.Audio.M4A,
    ContentType.Audio.FLAC,
    ContentType.Audio.OGG,
    ContentType.Application.OGG, -> getPlatform() == ChatPlatform.ANDROID

    else -> false
}

fun contentTypeFromFileName(name: String): ContentType? {
    val extension = name.substringAfterLast('.').lowercase()
    return ContentType.fromFileExtension(extension).firstOrNull() ?: run {
        when (extension) {
            "opus" -> ContentType.Audio.OPUS
            "flac" -> ContentType.Audio.FLAC
            "wav" -> ContentType.Audio.WAV
            "m4a" -> ContentType.Audio.M4A
            else -> null
        }
    }
}