package nand.apps.chat.repo

import kotlinx.coroutines.flow.Flow
import nand.apps.chat.model.file.ChatFileTransfer
import nand.apps.chat.model.group.ChatGroupInvite
import nand.apps.chat.model.message.ChatMessage
import nand.apps.chat.model.message.UnreadMessageCounts
import nand.apps.chat.model.uid.ContactUid
import nand.apps.chat.model.uid.MessageUid
import nand.apps.chat.model.uid.UserUid

interface ChatMessageRepo : ChatMessageRepoCallback {

    val onNewMessage: Flow<ChatMessage>
    val onMessagesChanged: Flow<ContactUid>
    val unreadObserver: Flow<Map<ContactUid, UnreadMessageCounts>>

    fun getMessages(conversationUid: ContactUid): List<ChatMessage>

    fun getMessage(conversationUid: ContactUid, messageUid: MessageUid): ChatMessage?

    fun getUnreadMessageCounts(conversationUid: ContactUid): UnreadMessageCounts

    fun findMessage(conversationUid: ContactUid, senderUid: ContactUid, textHash: String, timestamp: Long): ChatMessage?

    fun createMessage(text: String, conversationUid: ContactUid): ChatMessage

    fun sendMessage(message: ChatMessage): ChatMessage?

    fun addMessage(message: ChatMessage)

    fun updateMessage(message: ChatMessage, localOnly: Boolean, oldUid: MessageUid = message.uid)

    fun removeMessage(message: ChatMessage, localOnly: Boolean)

    fun removeMessages(conversationUid: ContactUid, isRemovingContact: Boolean)

    fun removeGroupInvite(groupInvite: ChatGroupInvite)

    fun markMessagesRead(conversationUid: ContactUid, messages: List<ChatMessage>)

    fun addReaction(message: ChatMessage, reaction: String)

    fun setReactions(senderUid: UserUid, message: ChatMessage, reactions: Set<String>, localOnly: Boolean)

    fun addFileTransferMessage(transfer: ChatFileTransfer, isAutoAccepted: Boolean)

    fun markConversationRead(conversationUid: ContactUid)
}