package nand.apps.chat.repo

import kotlinx.coroutines.flow.Flow
import nand.apps.chat.model.contact.ChatContact
import nand.apps.chat.model.group.*
import nand.apps.chat.model.uid.ContactUid
import nand.apps.chat.model.uid.FriendRequestUid
import nand.apps.chat.model.uid.GroupUid
import nand.apps.chat.model.uid.UserUid
import nand.apps.chat.model.user.FriendRequest
import nand.apps.chat.model.user.UserData

interface ContactRepo : ContactRepoCallback {

    val friendObserver: Flow<Map<UserUid, UserData>>
    val groupObserver: Flow<Map<GroupUid, ChatGroupData>>
    val friendRequestObserver: Flow<Map<FriendRequestUid, FriendRequest>>

    val onFriendRequest: Flow<FriendRequest>
    val onGroupInvite: Flow<ChatGroupInvite>

    val self: UserData

    fun getFriends(): List<UserData>

    fun getGroups(): List<ChatGroupData>

    fun getOnlineFriends(): List<UserData>

    fun getOnlineGroups(): List<ChatGroupData>

    fun getOfflineGroups(): List<ChatGroupData>

    fun getFriend(uid: UserUid): UserData?

    fun getFriendRequest(uid: FriendRequestUid): FriendRequest?

    fun getGroup(uid: GroupUid): ChatGroupData?

    fun getContact(uid: ContactUid, userGroup: ChatGroupData? = null): ChatContact?

    fun getContact(uid: String, userGroup: ChatGroupData? = null): ChatContact?

    fun getRandomStatusMessage(): String

    fun isPeerBlocked(groupUid: GroupUid, peerUid: UserUid): Boolean

    fun sendFriendRequest(name: String, address: String, message: String): UserData?

    fun acceptFriendRequest(friendRequest: FriendRequest): UserData?

    fun declineFriendRequest(friendRequest: FriendRequest): Boolean

    fun removeFriend(friendUid: UserUid): Boolean

    fun setFriendAlias(friendUid: UserUid, alias: String?)

    fun createGroup(title: String, type: ChatGroupType, visibility: ChatGroupVisibility): ChatGroupData?

    fun setGroupTitle(groupUid: GroupUid, title: String)

    fun setGroupDescription(groupUid: GroupUid, description: String)

    fun setGroupDescriptionLocked(groupUid: GroupUid, isLocked: Boolean)

    fun setGroupPassword(groupUid: GroupUid, password: String?)

    fun setGroupNickname(groupUid: GroupUid, name: String)

    fun setGroupPeerLimit(groupUid: GroupUid, peerLimit: Int)

    fun setGroupTalkState(groupUid: GroupUid, talkState: ChatGroupTalkState)

    fun setGroupVisibility(groupUid: GroupUid, visibility: ChatGroupVisibility)

    fun setGroupRole(groupUid: GroupUid, peerUid: UserUid, role: ChatGroupRole)

    fun kickGroupPeer(groupUid: GroupUid, peerUid: UserUid)

    fun setGroupPeerBlocked(groupUid: GroupUid, peerUid: UserUid, block: Boolean)

    fun joinGroup(groupName: String?, groupUid: GroupUid, selfNickname: String?, password: String?): ChatGroupData?

    fun sendGroupInvite(friendUid: UserUid, groupUid: GroupUid): Boolean

    fun acceptGroupInvite(invite: ChatGroupInvite): ChatGroupData?

    fun removeGroupInvite(invite: ChatGroupInvite): Boolean

    fun removeGroup(groupUid: GroupUid, partingMessage: String?)

    fun setContactMuted(uid: ContactUid, isMuted: Boolean)
}