package nand.apps.chat.repo

import nand.apps.chat.engine.ChatEngineCallback
import nand.apps.chat.model.group.*
import nand.apps.chat.model.net.ConnectionProtocol
import nand.apps.chat.model.uid.GroupUid
import nand.apps.chat.model.uid.UserUid
import nand.apps.chat.model.user.ChatClientVersion
import nand.apps.chat.model.user.FriendRequest
import nand.apps.chat.model.user.UserData
import nand.apps.chat.model.user.UserStatus

interface ContactRepoCallback : ChatEngineCallback {

    fun onFriendName(friendUid: UserUid, name: String): Boolean

    fun onFriendStatus(friendUid: UserUid, status: UserStatus): Boolean

    fun onFriendStatusMessage(friendUid: UserUid, message: String): Boolean

    fun onFriendClientVersion(friendUid: UserUid, version: ChatClientVersion): Boolean

    fun onFriendConnectionProtocol(friendUid: UserUid, protocol: ConnectionProtocol)

    fun onUserAvatarChanged(uid: UserUid, groupUid: GroupUid?, avatarPath: String?)

    fun onFriendRequest(request: FriendRequest)

    fun onGroupInvite(invite: ChatGroupInvite)

    fun onGroupAdded(group: ChatGroupData)

    fun onGroupConnected(groupUid: GroupUid, groupData: ChatGroupData? = null, fetchPeers: Boolean = false)

    fun onGroupErrorState(groupUid: GroupUid, error: ChatGroupErrorType)

    fun onGroupTitle(groupUid: GroupUid, title: String): Boolean

    fun onGroupDescription(groupUid: GroupUid, description: String): Boolean

    fun onGroupDescriptionLocked(groupUid: GroupUid, isLocked: Boolean): Boolean

    fun onGroupPassword(groupUid: GroupUid, password: String?): Boolean

    fun onGroupVisibility(groupUid: GroupUid, visibility: ChatGroupVisibility): Boolean

    fun onGroupPeerLimit(groupUid: GroupUid, peerLimit: Int): Boolean

    fun onGroupTalkState(groupUid: GroupUid, talkState: ChatGroupTalkState): Boolean

    fun onGroupAvatar(groupUid: GroupUid, avatarUri: String?): Boolean

    fun onGroupUsersChanged(groupUid: GroupUid): Boolean

    fun onGroupPeerJoin(groupUid: GroupUid, peer: UserData, isSelf: Boolean): Boolean

    fun onGroupPeerExit(groupUid: GroupUid, peerUid: UserUid): Boolean

    fun onGroupPeerName(groupUid: GroupUid, peerUid: UserUid, name: String): Boolean

    fun onGroupPeerStatus(groupUid: GroupUid, peerUid: UserUid, status: UserStatus): Boolean

    fun onGroupPeerClientVersion(groupUid: GroupUid, peerUid: UserUid, version: ChatClientVersion): Boolean

    fun onGroupPeerKicked(groupUid: GroupUid, sourceUid: UserUid, targetUid: UserUid)

    fun onGroupPeerRoleChanged(groupUid: GroupUid, sourceUid: UserUid, targetUid: UserUid, role: ChatGroupRole)
}