package nand.apps.chat.repo

import kotlinx.coroutines.flow.Flow
import nand.apps.chat.engine.ChatEngineCallback
import nand.apps.chat.model.net.ConnectionProtocol
import nand.apps.chat.model.settings.profile.ProfileSettingsData
import nand.apps.chat.model.uid.ContactUid
import nand.apps.chat.model.uid.UserUid
import nand.apps.chat.model.user.ChatClientVersion
import nand.apps.chat.model.user.UserData
import nand.apps.chat.model.user.UserStatus
import okio.Path

interface SelfProfileRepo : ChatEngineCallback {

    val self: UserData
    val selfObserver: Flow<UserData>
    val settings: ProfileSettingsData
    val profilePath: Path
    val profileName: String
    val clientVersion: ChatClientVersion

    fun setCurrentDisplayName(name: String)

    fun setCurrentAvatarUri(path: String?)

    fun setAutoAway(isAutoAway: Boolean, actualStatus: UserStatus)

    fun setSelfStatus(status: UserStatus)

    fun updateSettings(settings: ProfileSettingsData)

    fun onSelfConnectionProtocol(protocol: ConnectionProtocol)

    fun getSelfUid(conversationUid: ContactUid? = null): UserUid

    fun getSelfAddress(): String

    fun resetSelfAddress()
}