package nand.apps.chat.util

import kotlin.contracts.ExperimentalContracts
import kotlin.contracts.contract

@OptIn(ExperimentalContracts::class)
inline fun <T> List<T>.fastForEach(action: (T) -> Unit) {
    contract { callsInPlace(action) }
    for (index in indices) {
        val item = get(index)
        action(item)
    }
}

fun <T> MutableList<T>.addAll(vararg items: T) = addAll(items.toList())

fun <T> MutableList<T>.replaceAll(block: (T) -> T) {
    val iterator = listIterator()
    while (iterator.hasNext()) {
        iterator.set(block(iterator.next()))
    }
}

fun <T> MutableList<T>.copyAndClear() = toList().also { clear() }

inline fun <T> MutableList<T>.clearAndIterate(action: (T) -> Unit) {
    if (isNotEmpty()) {
        copyAndClear().fastForEach(action)
    }
}

fun <K, V> Iterable<Pair<K, V>>.toMutableMap() = toMap(LinkedHashMap())

fun <K, V> MutableMap<K, V>.setAll(from: Map<K, V>) {
    clear()
    putAll(from)
}

fun <K, V> MutableMap<K, V>.removeIf(block: (K, V) -> Boolean) {
    val removeKeys = mutableListOf<K>()
    forEach { (key, value) ->
        if (block(key, value)) removeKeys += key
    }
    removeKeys.forEach { remove(it) }
}

fun <K, V> MutableMap<K, V>.removeFirst() = remove(keys.first())