package nand.apps.chat.util

import com.fleeksoft.ksoup.Ksoup
import com.fleeksoft.ksoup.nodes.Document
import io.ktor.client.*
import io.ktor.client.plugins.*
import io.ktor.client.request.*
import io.ktor.client.statement.*
import kotlinx.serialization.json.Json
import kotlinx.serialization.json.JsonObject

val URL_REGEX = Regex("https?://([\\w.\\-]{1,255}\\.[a-z]{1,32})\\b[-\\w()@$:;%+.,~#?&/=*']*")
val HOST_CHARS_REGEX = Regex("[\\w.\\-:]{1,255}")

fun CharSequence.isUrl() = URL_REGEX.matches(this)

fun CharSequence.getHostName() = URL_REGEX.matchEntire(this)?.groupValues?.getOrNull(1)

fun String.getUrlFileName() = substringAfterLast('/').substringBefore('?')

fun CharSequence.findUrls() = URL_REGEX.findAll(this).map { it.value }.toList()

fun createDefaultHttpClient() = HttpClient {
    install(UserAgent) {
        // Generic web browser user agent to help avoid bot filters
        agent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/124.0.0.0 Safari/537.36"
    }
}

suspend fun HttpClient.getHtmlDocument(url: String): Document? {
    return try {
        Ksoup.parse(get(url).bodyAsText())
    } catch (ignore: Exception) {
        null
    }
}

suspend fun HttpClient.getJsonObject(url: String): JsonObject? {
    return try {
        Json.decodeFromString(get(url).bodyAsText())
    } catch (ignore: Exception) {
        null
    }
}