package nand.apps.chat.util

import kotlinx.coroutines.ExperimentalForInheritanceCoroutinesApi
import kotlinx.coroutines.flow.FlowCollector
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlin.reflect.KProperty

@OptIn(ExperimentalForInheritanceCoroutinesApi::class)
class MutableStateFlowDelegate<V>(initialValue: V) : StateFlow<V> {

    private val stateFlow = MutableStateFlow(initialValue)

    override val value get() = stateFlow.value
    override val replayCache get() = stateFlow.replayCache

    override suspend fun collect(collector: FlowCollector<V>) = stateFlow.collect(collector)

    operator fun getValue(thisRef: Any?, property: KProperty<*>): V {
        return stateFlow.value
    }

    operator fun setValue(thisRef: Any?, property: KProperty<*>, value: V): V {
        stateFlow.tryEmit(value)
        return value
    }
}