package nand.apps.chat.util

import kotlinx.coroutines.ExperimentalForInheritanceCoroutinesApi
import kotlinx.coroutines.flow.FlowCollector
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow

@OptIn(ExperimentalForInheritanceCoroutinesApi::class)
class MutableStateFlowList<E> : MutableList<E>, StateFlow<List<E>> {

    private val impl = mutableListOf<E>()
    private val stateFlow = MutableStateFlow(emptyList<E>())

    override val size get() = impl.size
    override val value get() = stateFlow.value
    override val replayCache get() = stateFlow.replayCache

    override fun clear() {
        if (isNotEmpty()) {
            impl.clear()
            emitState()
        }
    }

    override fun addAll(elements: Collection<E>) =
        impl.addAll(elements).also(::emitState)

    override fun addAll(index: Int, elements: Collection<E>) =
        impl.addAll(index, elements).also(::emitState)

    override fun add(index: Int, element: E) =
        impl.add(index, element).also { emitState() }

    override fun add(element: E) =
        impl.add(element).also(::emitState)

    override fun get(index: Int) = impl[index]

    override fun isEmpty() = impl.isEmpty()

    override fun iterator() = impl.iterator()

    override fun listIterator() = impl.listIterator()

    override fun listIterator(index: Int) = impl.listIterator(index)

    override fun removeAt(index: Int) =
        impl.removeAt(index).also { emitState() }

    override fun subList(fromIndex: Int, toIndex: Int) =
        impl.subList(fromIndex, toIndex)

    override fun set(index: Int, element: E) =
        impl.set(index, element).also { emitState() }

    override fun retainAll(elements: Collection<E>) =
        impl.retainAll(elements).also(::emitState)

    override fun removeAll(elements: Collection<E>) =
        impl.removeAll(elements).also(::emitState)

    override fun remove(element: E) =
        impl.remove(element).also(::emitState)

    override fun lastIndexOf(element: E) = impl.lastIndexOf(element)

    override fun indexOf(element: E) = impl.indexOf(element)

    override fun containsAll(elements: Collection<E>) = impl.containsAll(elements)

    override fun contains(element: E) = impl.contains(element)

    override suspend fun collect(collector: FlowCollector<List<E>>) = stateFlow.collect(collector)

    private fun emitState(listModified: Boolean = true) {
        if (listModified) stateFlow.tryEmit(impl.toList())
    }
}