import org.jetbrains.kotlin.gradle.dsl.JvmTarget

plugins {
    alias(libs.plugins.kotlinMultiplatform)
    alias(libs.plugins.kotlinSerialization)
    alias(libs.plugins.androidLibrary)
    alias(libs.plugins.jetbrainsCompose)
    alias(libs.plugins.jetbrainsComposeKotlin)
    alias(libs.plugins.buildConfig)
}

buildConfig {
    packageName = "nand.apps.chat.matrix"
    buildConfigField("TRIXNITY_VERSION", libs.trixnity.client.get().version)
}

kotlin {
    androidTarget {
        compilerOptions {
            jvmTarget = JvmTarget.JVM_11
        }
    }

    jvm("desktop")

    listOf(
        iosX64(),
        iosArm64(),
        iosSimulatorArm64()
    ).forEach { iosTarget ->
        iosTarget.binaries.framework {
            baseName = "Matrix"
            isStatic = true
        }
    }

    sourceSets {
        commonMain.dependencies {
            implementation(project(":libraries:core"))
            implementation(project(":libraries:audio"))
            implementation(compose.runtime)
            implementation(compose.components.resources)
            implementation(libs.kotlinx.coroutines.core)
            implementation(libs.kotlinx.serialization.core)
            implementation(libs.kotlinx.serialization.json)
            implementation(libs.ktor.client.core)
            implementation(libs.okio)
            implementation(libs.koin.core)
            implementation(libs.bundles.coil)
            implementation(libs.bundles.trixnity)
        }
    }
}

android {
    namespace = "chat.seers.lib.matrix"
    compileSdk = libs.versions.android.compileSdk.get().toInt()
    defaultConfig {
        minSdk = libs.versions.android.minSdk.get().toInt()
    }
    compileOptions {
        sourceCompatibility = JavaVersion.VERSION_11
        targetCompatibility = JavaVersion.VERSION_11
    }
}