package nand.apps.chat.engine.matrix

import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Job
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.FlowCollector
import kotlinx.coroutines.launch

class MatrixCoroutineScheduler(
    private val coroutineScope: CoroutineScope,
) {
    private val jobs = mutableListOf<Job>()

    fun shutdown() {
        jobs.forEach { it.cancel() }
        jobs.clear()
    }

    fun launch(block: suspend CoroutineScope.() -> Unit) = coroutineScope.launch(block = block)

    fun <T> collect(flow: Flow<T>, collector: FlowCollector<T>): Job {
        return coroutineScope.launch { flow.collect(collector) }.also { job ->
            jobs.removeAll { !it.isActive }
            jobs += job
        }
    }
}