package nand.apps.chat.engine.matrix.image

import coil3.ImageLoader
import coil3.Uri
import coil3.decode.DataSource
import coil3.decode.ImageSource
import coil3.fetch.FetchResult
import coil3.fetch.Fetcher
import coil3.fetch.SourceFetchResult
import coil3.request.Options
import nand.apps.chat.engine.matrix.MatrixChatEngine
import okio.Buffer
import org.koin.core.Koin

class MatrixImageFetcher(
    private val url: String,
    private val options: Options,
    private val chatEngine: MatrixChatEngine,
) : Fetcher {

    override suspend fun fetch(): FetchResult? {
        val data = chatEngine.getAvatarData(url) ?: return null
        return SourceFetchResult(
            source = ImageSource(
                source = Buffer().apply { write(data) },
                fileSystem = options.fileSystem,
            ),
            mimeType = null,
            dataSource = DataSource.MEMORY,
        )
    }

    class Factory(koin: Koin) : Fetcher.Factory<Uri> {

        private val chatEngine: MatrixChatEngine by koin.inject()

        override fun create(data: Uri, options: Options, imageLoader: ImageLoader): Fetcher? {
            val url = data.takeIf { it.scheme == "mxc" } ?: return null
            return MatrixImageFetcher(
                url = url.toString(),
                options = options,
                chatEngine = chatEngine,
            )
        }
    }
}