package nand.apps.chat.engine.matrix.profile

import kotlinx.serialization.json.Json
import nand.apps.chat.io.ChatDirectory
import nand.apps.chat.io.ChatFileSystem
import nand.apps.chat.log.getLogger
import nand.apps.chat.model.user.ChatProfileData
import okio.Path.Companion.toPath

class MatrixProfileRepo(
    private val fileSystem: ChatFileSystem,
) {

    private lateinit var profileDir: ChatDirectory
    private lateinit var profileName: String

    private val json = Json { ignoreUnknownKeys = true }
    private val logger = getLogger("MatrixProfileSaver")

    fun init(profile: ChatProfileData) {
        this.profileDir = fileSystem.getProfileDirectory(profile)
        this.profileName = profile.filePath.toPath().name
    }

    fun load(): MatrixProfileData? {
        if (profileName !in profileDir) return null

        val data = profileDir.readFromFile(profileName) ?: return null

        return try {
            json.decodeFromString(data.decodeToString())
        } catch (e: Exception) {
            logger.error("Failed to deserialize profile data", e)
            null
        }
    }

    fun save(profile: MatrixProfileData) {
        val data = try {
            json.encodeToString(profile).encodeToByteArray()
        } catch (e: Exception) {
            logger.error("Failed to serialize profile data", e)
            return
        }
        if (profileDir.writeToFile(profileName, data)) {
            logger.debug("Saved profile data to $profileName (${data.size} bytes)")
        } else {
            logger.error("Failed to save profile data to $profileName")
        }
    }
}