package nand.apps.mpeg

import nand.apps.audio.AudioFileReader
import kotlin.math.roundToLong

private const val NULL_PTR = 0L

class MpegFileReader(path: String) : AudioFileReader {

    private val ptr = MpegNative.openFileReader(path)
    private val info by lazy { if (isOpen) MpegNative.getFileInfo(ptr) else null }
    private val timeRatio by lazy { sampleRate.toDouble() * channels }

    override val isOpen get() = ptr != NULL_PTR
    override val timestamp get() = if (isOpen) (MpegNative.getSamplePosition(ptr) / timeRatio) else 0.0
    override val duration by lazy { info?.run { numSamples / timeRatio } ?: 0.0 }
    override val sampleCount by lazy { info?.numSamples ?: 0L }
    override val channels by lazy { info?.channels ?: 0 }
    override val sampleRate by lazy { info?.sampleRate ?: 0 }

    override fun seekTo(timestamp: Double): Boolean {
        return if (isOpen) MpegNative.seekTo(ptr, (timestamp * timeRatio).roundToLong()) else false
    }

    override fun read(buffer: ShortArray): Int {
        return when {
            isOpen -> MpegNative.readSamples(
                ptr = ptr,
                buffer = buffer,
            )
            else -> 0
        }
    }

    override fun close() {
        if (isOpen) MpegNative.closeReader(ptr)
    }

    fun finalize() = close()
}