package nand.apps.opus

import nand.apps.audio.AudioFileReader
import kotlin.math.roundToLong

class OpusFileReader private constructor() : OpusFileStream(), AudioFileReader {

    override val sampleCount by lazy {
        if (isOpen) OpusNative.getSampleCount(ptr) else -1
    }

    override val timestamp get() = if (isOpen) (OpusNative.getSamplePosition(ptr).toDouble() / sampleRate) else 0.0

    override val duration by lazy {
        if (isOpen) sampleCount.toDouble() / sampleRate else 0.0
    }

    // XXX - The reader always returns samples in stereo, even if the file is mono
    override val channels = DEFAULT_CHANNEL_COUNT
    override val sampleRate = OPUS_SAMPLE_RATE

    constructor(path: String) : this() {
        ptr = OpusNative.openFileReader(path)
    }

    constructor(data: ByteArray) : this() {
        ptr = OpusNative.openMemoryReader(data)
    }

    override fun seekTo(timestamp: Double): Boolean {
        return if (isOpen) OpusNative.seekTo(ptr, (timestamp * sampleRate).roundToLong()) else false
    }

    fun read(buffer: ShortArray, offset: Int, length: Int): Int {
        return when {
            isOpen -> OpusNative.readSamples(
                ptr = ptr,
                buffer = buffer,
                offset = offset,
                length = length,
            ) * channels
            else -> 0
        }
    }

    override fun read(buffer: ShortArray) = read(
        buffer = buffer,
        offset = 0,
        length = buffer.size,
    )

    override fun closeImpl() {
        OpusNative.closeReader(ptr)
    }
}