package nand.apps.opus

import nand.apps.audio.AudioFileWriter

class OpusFileWriter(
    val path: String,
    val sampleRate: Int,
    override val channels: Int,
    val bitRate: Int,
    val metadata: OpusFileMetadata? = null,
) : OpusFileStream(), AudioFileWriter {

    init {
        val metadataPtr = metadata?.let { OpusNative.createMetadata() }?.also { metadataPtr ->
            metadata.title?.let { OpusNative.writeMetadata(metadataPtr, "title", it.encodeToByteArray()) }
            metadata.artist?.let { OpusNative.writeMetadata(metadataPtr, "artist", it.encodeToByteArray()) }
        } ?: NULL_PTR
        ptr = OpusNative.openFileWriter(
            path = path,
            sampleRate = sampleRate,
            channels = channels,
            bitRate = bitRate,
            metadataPtr = metadataPtr,
        )
        OpusNative.destroyMetadata(metadataPtr)
    }

    override fun write(buffer: ShortArray, offset: Int, length: Int): Int {
        return when {
            isOpen -> OpusNative.writeSamples(
                ptr = ptr,
                buffer = buffer,
                offset = offset,
                samplesPerChannel = length / channels,
            )
            else -> 0
        }
    }

    override fun closeImpl() {
        OpusNative.closeWriter(ptr)
    }
}