package nand.apps.tox.net

import kotlinx.serialization.SerialName
import kotlinx.serialization.Serializable
import nand.apps.chat.util.toLongOrZero

@Serializable
data class ToxBootstrapNode(
    val ipv4: String,
    val ipv6: String,
    val maintainer: String,
    val version: String,
    val motd: String,
    val location: String,
    @SerialName("port") val udpPort: Int,
    @SerialName("tcp_ports") val tcpPorts: List<Int>,
    @SerialName("public_key") val publicKey: String,
    @SerialName("status_udp") val isUdpOnline: Boolean,
    @SerialName("status_tcp") val isTcpOnline: Boolean,
) {
    
    val isOnline get() = isUdpOnline || isTcpOnline
    val ip get() = if (ipv4.isValidIpAddress()) ipv4 else ipv6

    val formattedVersion by lazy {
        var rawVersion = version.toLongOrZero() % 1000000000L
        if (rawVersion > 0) {
            val versionMajor = rawVersion / 1000000L
            rawVersion -= versionMajor * 1000000L
            val versionMinor = rawVersion / 1000L
            val versionPatch = rawVersion % 1000L
            "v$versionMajor.$versionMinor.$versionPatch"
        } else {
            ""
        }
    }
}

fun String.isValidIpAddress() = isNotBlank() && this != "-" && this != "NONE"